# -*- coding: utf-8 -*-
from abjad import *


def test_quantizationtools_QuantizationJob___call___01():

    job_id = 1
    definition = {
        2: {
            2: {
                2: None
                },
            3: None
            },
        5: None
        }
    search_tree = quantizationtools.UnweightedSearchTree(definition)
    q_event_proxies = [
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent(0, ['A'], index=1), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((1, 5), ['B'], index=2), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((1, 4), ['C'], index=3), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((1, 3), ['D'], index=4), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((2, 5), ['E'], index=5), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((1, 2), ['F'], index=6), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((3, 5), ['G'], index=7), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((2, 3), ['H'], index=8), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((3, 4), ['I'], index=9), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent((4, 5), ['J'], index=10), 0, 1),
        quantizationtools.QEventProxy(
            quantizationtools.SilentQEvent(1, ['K'], index=11), 0, 1)
        ]
    job = quantizationtools.QuantizationJob(
        job_id, search_tree, q_event_proxies)
    job()

    assert len(job.q_grids) == 10

    rtm_formats = [q_grid.root_node.rtm_format for q_grid in job.q_grids]
    rtm_formats.sort(reverse=True)

    assert rtm_formats == [
        '1',
        '(1 (1 1))',
        '(1 (1 1 1 1 1))',
        '(1 ((1 (1 1)) (1 (1 1))))',
        '(1 ((1 (1 1)) (1 (1 1 1))))',
        '(1 ((1 (1 1 1)) (1 (1 1))))',
        '(1 ((1 (1 1 1)) (1 (1 1 1))))',
        '(1 ((1 (1 1 1)) (1 ((1 (1 1)) (1 (1 1))))))',
        '(1 ((1 ((1 (1 1)) (1 (1 1)))) (1 (1 1 1))))',
        '(1 ((1 ((1 (1 1)) (1 (1 1)))) (1 ((1 (1 1)) (1 (1 1))))))'
        ], rtm_formats



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import selectiontools
from abjad.tools.topleveltools import attach


def make_skips(
    written_duration,
    multiplied_durations,
    ):
    '''Make `written_duration` skips with `multiplied_durations`:

    ::

        >>> scoretools.make_skips(
        ...     Duration(1, 4), [(1, 2), (1, 3), (1, 4), (1, 5)])
        Selection([Skip('s4 * 2'), Skip('s4 * 4/3'), Skip('s4 * 1'), Skip('s4 * 4/5')])

    Useful for making invisible layout voices.

    Returns selection.
    '''
    from abjad.tools import scoretools

    # initialize skips and written duration
    skips = []
    written_duration = durationtools.Duration(written_duration)

    # make skips
    for multiplied_duration in multiplied_durations:
        multiplied_duration = durationtools.Duration(multiplied_duration)
        skip = scoretools.Skip(written_duration)
        multiplier = multiplied_duration / written_duration
        attach(multiplier, skip)
        skips.append(skip)

    # return skips
    skips = selectiontools.Selection(skips)
    return skips



# -*- coding: utf-8 -*-
import abc
from abjad.tools.abctools.AbjadObject import AbjadObject


class BoundedObject(AbjadObject):
    r'''Bounded object mix-in.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### PUBLIC PROPERTIES ###

    @property
    def is_closed(self):
        r'''Is true when left closed and right closed. Otherwise false.

        Returns true or false.
        '''
        return self.is_left_closed and self.is_right_closed

    @property
    def is_half_closed(self):
        r'''Is true when left closed xor right closed.

        Returns true or false.
        '''
        return not self.is_left_closed == self.is_right_closed

    @property
    def is_half_open(self):
        r'''Is true when left and right open are not the same. Otherwise false.

        Return boolean.
        '''
        return not self.is_left_open == self.is_right_open

    @property
    def is_left_closed(self):
        r'''Is true when left closed. Otherwise false.

        Returns true or false.
        '''
        return self._is_left_closed

    @is_left_closed.setter
    def is_left_closed(self, is_left_closed):
        assert isinstance(is_left_closed, bool), is_left_closed
        self._is_left_closed = is_left_closed

    @property
    def is_left_open(self):
        r'''Is true when left open. Otherwise false.

        Returns true or false.
        '''
        return not self.is_left_closed

    @is_left_open.setter
    def is_left_open(self, is_left_open):
        assert isinstance(is_left_open, bool), is_left_open
        self._is_left_closed = not is_left_open

    @property
    def is_open(self):
        r'''Is true when left or right open. Otherwise false.

        Returns true or false.
        '''
        return not self.is_left_closed and not self.is_right_closed

    @property
    def is_right_closed(self):
        r'''Is true when right closed. Otherwise false.

        Returns true or false.
        '''
        return self._is_right_closed

    @is_right_closed.setter
    def is_right_closed(self, is_right_closed):
        assert isinstance(is_right_closed, bool), is_right_closed
        self._is_right_closed = is_right_closed

    @property
    def is_right_open(self):
        r'''Is true when right open. Otherwise false.

        Returns true or false.
        '''
        return not self.is_right_closed

    @is_right_open.setter
    def is_right_open(self, is_right_open):
        assert isinstance(is_right_open, bool), is_right_open
        self._is_right_closed = not is_right_open



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools.scoretools.Container import Container


class FixedDurationContainer(Container):
    r'''A fixed-duration container.

    ::

        >>> container = scoretools.FixedDurationContainer(
        ...     (3, 8), "c'8 d'8 e'8")
        >>> show(container) # doctest: +SKIP

    ..  doctest::

        >>> print(format(container))
        {
            c'8
            d'8
            e'8
        }


    Fixed-duration containers extend container behavior with format-time
    checking against a user-specified target duration.

    Returns fixed-duration container.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Containers'

    __slots__ = (
        '_target_duration',
        )

    ### INITIALIZER ###

    def __init__(self, target_duration=None, music=None, **keywords):
        Container.__init__(self, music=music, **keywords)
        target_duration = target_duration or durationtools.Duration(1, 4)
        target_duration = durationtools.Duration(target_duration)
        assert 0 < target_duration
        self._target_duration = target_duration

    ### SPECIAL METHODS ###

    def __repr__(self):
        r'''Gets interpreter representation of fixed-duration container.

        Returns string.
        '''
        result = '{}({!r}, {})'
        result = result.format(
            type(self).__name__,
            self.target_duration,
            list(self[:]),
            )
        return result

    ### PRIVATE METHODS ###

    def _check_duration(self):
        from abjad.tools import indicatortools
        preprolated_duration = self._contents_duration
        if preprolated_duration < self.target_duration:
            raise UnderfullContainerError
        if self.target_duration < preprolated_duration:
            raise OverfullContainerError

    def _get_lilypond_format(self):
        self._check_duration()
        return self._format_component()

    ### PUBLIC PROPERTIES ###

    @property
    def is_full(self):
        r'''Is true when preprolated duration equals target duration. Otherwise
        false.

        Returns true or false.
        '''
        return self._preprolated_duration == self.target_duration

    @property
    def is_misfilled(self):
        r'''Is true when preprolated duration does not equal target duration.
        Otherwise false.

        Returns true or false.
        '''
        return not self.is_full

    @property
    def is_overfull(self):
        r'''Is true when preprolated duration is greater than target duration.
        Otherwise false.

        Returns true or false.
        '''
        return self.target_duration < self._preprolated_duration

    @property
    def is_underfull(self):
        r'''Is true when preprolated duration is less than target duration.
        Otherwise false.

        Returns true or false.
        '''
        return self._preprolated_duration < self.target_duration

    @property
    def target_duration(self):
        r'''Gets and sets target duration of fixed-duration container.

        Returns duration.
        '''
        return self._target_duration

    @target_duration.setter
    def target_duration(self, target_duration):
        target_duration = durationtools.Duration(target_duration)
        assert 0 < target_duration
        self._target_duration = target_duration



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchClassSet_transpose_01():

    pitch_class_set_1 = pitchtools.PitchClassSet([
        pitchtools.NamedPitchClass('c'),
        pitchtools.NamedPitchClass('d'),
        pitchtools.NamedPitchClass('e'),])

    pitch_class_set_2 = pitchtools.PitchClassSet([
        pitchtools.NamedPitchClass('df'),
        pitchtools.NamedPitchClass('ef'),
        pitchtools.NamedPitchClass('f'),])

    minor_second_ascending = pitchtools.NamedInterval('minor', 2)
    assert pitch_class_set_1.transpose(minor_second_ascending) == \
        pitch_class_set_2

    major_seventh_descending = pitchtools.NamedInterval('major', -7)
    assert pitch_class_set_1.transpose(major_seventh_descending) == \
        pitch_class_set_2

    minor_second_descending = pitchtools.NamedInterval('minor', -2)
    assert pitch_class_set_2.transpose(minor_second_descending) == \
        pitch_class_set_1

    major_seventh_ascending = pitchtools.NamedInterval('major', 7)
    assert pitch_class_set_2.transpose(major_seventh_ascending) == \
        pitch_class_set_1


def test_pitchtools_PitchClassSet_transpose_02():

    pitch_class_set = pitchtools.PitchClassSet([1, 2, 5])
    assert pitch_class_set.transpose(0) == pitchtools.PitchClassSet([1, 2, 5])
    assert pitch_class_set.transpose(1) == pitchtools.PitchClassSet([2, 3, 6])
    assert pitch_class_set.transpose(2) == pitchtools.PitchClassSet([3, 4, 7])
    assert pitch_class_set.transpose(3) == pitchtools.PitchClassSet([4, 5, 8])
    assert pitch_class_set.transpose(4) == pitchtools.PitchClassSet([5, 6, 9])
    assert pitch_class_set.transpose(5) == pitchtools.PitchClassSet([6, 7, 10])
    assert pitch_class_set.transpose(6) == pitchtools.PitchClassSet([7, 8, 11])
    assert pitch_class_set.transpose(7) == pitchtools.PitchClassSet([8, 9, 0])
    assert pitch_class_set.transpose(8) == pitchtools.PitchClassSet([9, 10, 1])
    assert pitch_class_set.transpose(9) == pitchtools.PitchClassSet([10, 11, 2])
    assert pitch_class_set.transpose(10) == pitchtools.PitchClassSet([11, 0, 3])
    assert pitch_class_set.transpose(11) == pitchtools.PitchClassSet([0, 1, 4])



# -*- coding: utf-8 -*-
import copy
import inspect
import pytest
from abjad.tools import abjadbooktools
from abjad.tools import documentationtools
from abjad.tools import scoretools
from abjad.tools import systemtools


ignored_classes = (
    abjadbooktools.CodeBlock,
    abjadbooktools.CodeOutputProxy,
    abjadbooktools.GraphvizOutputProxy,
    abjadbooktools.LilyPondOutputProxy,
    systemtools.StorageFormatAgent,
    systemtools.FormatSpecification,
    )

classes = documentationtools.list_all_abjad_classes(
    ignored_classes=ignored_classes,
    )


@pytest.mark.parametrize('class_', classes)
def test_abjad___copy___01(class_):
    r'''All concrete classes with a storage format can copy.
    '''
    if (
        '_get_storage_format_specification' not in dir(class_) or
        '_get_format_specification' not in dir(class_)
        ): 
        return
    if inspect.isabstract(class_):
        return
    instance_one = class_()
    instance_two = copy.copy(instance_one)
    instance_one_format = format(instance_one, 'storage')
    instance_two_format = format(instance_two, 'storage')
    if not issubclass(class_, scoretools.Container):
        assert instance_one_format == instance_two_format
    # TODO: eventually this second asset should also pass
    #assert instance_one == instance_two



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_Accidental_symbolic_accidental_string_01():

    assert pitchtools.Accidental('s').symbolic_string == '#'
    assert pitchtools.Accidental('ss').symbolic_string == '##'
    assert pitchtools.Accidental('f').symbolic_string == 'b'
    assert pitchtools.Accidental('ff').symbolic_string == 'bb'
    assert pitchtools.Accidental('').symbolic_string == ''


def test_pitchtools_Accidental_symbolic_accidental_string_02():

    assert pitchtools.Accidental('ff').symbolic_string == 'bb'
    assert pitchtools.Accidental('tqf').symbolic_string == 'b~'
    assert pitchtools.Accidental('f').symbolic_string == 'b'
    assert pitchtools.Accidental('qf').symbolic_string == '~'
    assert pitchtools.Accidental('').symbolic_string == ''
    assert pitchtools.Accidental('qs').symbolic_string == '+'
    assert pitchtools.Accidental('s').symbolic_string == '#'
    assert pitchtools.Accidental('tqs').symbolic_string == '#+'
    assert pitchtools.Accidental('ss').symbolic_string == '##'



# -*- coding: utf-8 -*-
from abjad import *


def test_agenttools_InspectionAgent__select_vertical_moment_at_01():

    score = Score([])
    tuplet = scoretools.FixedDurationTuplet(Duration(4, 8), [])
    tuplet.extend("d''8 c''8 b'8")
    score.append(Staff([tuplet]))
    staff_group = StaffGroup([])
    staff_group.context_name = 'PianoStaff'
    staff_group.append(Staff("a'4 g'4"))
    staff_group.append(Staff("f'8 e'8 d'8 c'8"))
    clef = Clef('bass')
    attach(clef, staff_group[1])
    score.append(staff_group)

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tweak text #tuplet-number::calc-fraction-text
                \times 4/3 {
                    d''8
                    c''8
                    b'8
                }
            }
            \new PianoStaff <<
                \new Staff {
                    a'4
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    f'8
                    e'8
                    d'8
                    c'8
                }
            >>
        >>
        '''
        )

    def staff_group_moment(offset):
        return inspect_(staff_group).get_vertical_moment_at(offset)

    moment = staff_group_moment(Offset(0, 8))
    assert moment.leaves == (staff_group[0][0], staff_group[1][0])

    moment = staff_group_moment(Offset(1, 8))
    assert moment.leaves == (staff_group[0][0], staff_group[1][1])

    moment = staff_group_moment(Offset(2, 8))
    assert moment.leaves == (staff_group[0][1], staff_group[1][2])

    moment = staff_group_moment(Offset(3, 8))
    assert moment.leaves == (staff_group[0][1], staff_group[1][3])

    moment = staff_group_moment(Offset(99, 8))
    assert moment.leaves == ()


def test_agenttools_InspectionAgent__select_vertical_moment_at_02():

    score = Score([])
    tuplet = scoretools.FixedDurationTuplet(Duration(4, 8), [])
    tuplet.extend("d''8 c''8 b'8")
    score.append(Staff([tuplet]))
    staff_group = StaffGroup([])
    staff_group.context_name = 'PianoStaff'
    staff_group.append(Staff("a'4 g'4"))
    staff_group.append(Staff("f'8 e'8 d'8 c'8"))
    clef = Clef('bass')
    attach(clef, staff_group[1])
    score.append(staff_group)

    assert format(score) == stringtools.normalize(
        r'''
        \new Score <<
            \new Staff {
                \tweak text #tuplet-number::calc-fraction-text
                \times 4/3 {
                    d''8
                    c''8
                    b'8
                }
            }
            \new PianoStaff <<
                \new Staff {
                    a'4
                    g'4
                }
                \new Staff {
                    \clef "bass"
                    f'8
                    e'8
                    d'8
                    c'8
                }
            >>
        >>
        '''
        )

    def scorewide_vertical_moment(offset):
        return inspect_(score).get_vertical_moment_at(offset)

    moment = scorewide_vertical_moment(Offset(0, 8))
    assert moment.leaves == (
        score[0][0][0],
        staff_group[0][0],
        staff_group[1][0],
        )

    moment = scorewide_vertical_moment(Offset(1, 8))
    assert moment.leaves == (
        score[0][0][0],
        staff_group[0][0],
        staff_group[1][1],
        )

    moment = scorewide_vertical_moment(Offset(2, 8))
    assert moment.leaves == (
        score[0][0][1],
        staff_group[0][1],
        staff_group[1][2],
        )

    moment = scorewide_vertical_moment(Offset(3, 8))
    assert moment.leaves == (
        score[0][0][2],
        staff_group[0][1],
        staff_group[1][3],
        )

    moment = scorewide_vertical_moment(Offset(99, 8))
    assert moment.leaves == ()



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_pitchtools_NumberedPitchClass___slots___01():
    r'''Numbered pitch-classes are immutable.
    '''

    numbered_pitch_class = pitchtools.NumberedPitchClass(1)
    assert pytest.raises(AttributeError, "numbered_pitch_class.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools import tonalanalysistools


def test_tonalanalysistools_ChordSuspension___eq___01():

    chord_suspension = tonalanalysistools.ChordSuspension(4, 3)
    u = tonalanalysistools.ChordSuspension(4, 3)
    voice = tonalanalysistools.ChordSuspension(2, 1)

    assert      chord_suspension == chord_suspension
    assert      chord_suspension == u
    assert not chord_suspension == voice

    assert      u == chord_suspension
    assert      u == u
    assert not u == voice

    assert not voice == chord_suspension
    assert not voice == u
    assert      voice == voice



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_KeySignature___repr___01():
    r'''Key signature returns nonempty string repr.
    '''

    result = repr(KeySignature('g', 'major'))
    assert isinstance(result, str) and 0 < len(result)



# -*- coding: utf-8 -*-
from abjad import *


def test_selectiontools_LogicalTie_leaves_01():

    staff = Staff("c' ~ c'16")

    assert inspect_(staff[0]).get_logical_tie().leaves == tuple(staff[:])


def test_selectiontools_LogicalTie_leaves_02():

    staff = Staff("c'")

    assert inspect_(staff[0]).get_logical_tie().leaves == (staff[0], )



# -*- coding: utf-8 -*-
from abjad.tools import scoretools
from abjad.tools import indicatortools
from abjad.tools import scoretools
from abjad.tools import pitchtools
from abjad.tools.topleveltools import iterate


def transpose_from_sounding_pitch_to_written_pitch(argument):
    r'''Transpose notes and chords in `argument` from sounding pitch
    to written pitch:

    ::

        >>> staff = Staff("<c' e' g'>4 d'4 r4 e'4")
        >>> clarinet = instrumenttools.ClarinetInBFlat()
        >>> attach(clarinet, staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { "Clarinet in B-flat" }
            \set Staff.shortInstrumentName = \markup { "Cl. in B-flat" }
            <c' e' g'>4
            d'4
            r4
            e'4
        }

    ::

        >>> instrumenttools.transpose_from_sounding_pitch_to_written_pitch(staff)
        >>> show(staff) # doctest: +SKIP

    ..  doctest::

        >>> f(staff)
        \new Staff {
            \set Staff.instrumentName = \markup { "Clarinet in B-flat" }
            \set Staff.shortInstrumentName = \markup { "Cl. in B-flat" }
            <d' fs' a'>4
            e'4
            r4
            fs'4
        }

    Returns none.
    '''
    from abjad.tools import instrumenttools
    prototype = (scoretools.Note, scoretools.Chord)
    for note_or_chord in iterate(argument).by_class(prototype):
        instrument = note_or_chord._get_effective(instrumenttools.Instrument)
        if not instrument:
            continue
        sounding_pitch = instrument.sounding_pitch_of_written_middle_c
        interval = pitchtools.NamedPitch('C4') - sounding_pitch
        interval *= -1
        if isinstance(note_or_chord, scoretools.Note):
            pitch = note_or_chord.written_pitch
            pitch = interval.transpose(pitch)
            note_or_chord.written_pitch = pitch
        elif isinstance(note_or_chord, scoretools.Chord):
            pitches = [
                interval.transpose(pitch)
                for pitch in note_or_chord.written_pitches
                ]
            note_or_chord.written_pitches = pitches



# -*- coding: utf-8 -*-
from abjad.tools.topleveltools import parse


def make_mozart_measure(measure_dict):
    r'''Makes Mozart measure.
    '''

    # parse the contents of a measure definition dictionary
    # wrap the expression to be parsed inside a LilyPond { } block
    treble = parse('{{ {} }}'.format(measure_dict['t']))
    bass = parse('{{ {} }}'.format(measure_dict['b']))
    return treble, bass


# -*- coding: utf-8 -*-
from abjad import *
from abjad.tools.indicatortools import LilyPondComment


def test_indicatortools_LilyPondComment___repr___01():
    r'''Repr of unattached comment is evaluable.
    '''

    comment_1 = indicatortools.LilyPondComment('foo')
    comment_2 = eval(repr(comment_1))

    assert isinstance(comment_1, indicatortools.LilyPondComment)
    assert isinstance(comment_2, indicatortools.LilyPondComment)
    assert comment_1 == comment_2



# -*- coding: utf-8 -*-
from abjad.tools import markuptools
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import set_


def make_reference_manual_lilypond_file(music=None, **keywords):
    r'''Makes reference manual LilyPond file.

        >>> score = Score([Staff('c d e f')])
        >>> lilypond_file = \
        ...     documentationtools.make_reference_manual_lilypond_file(score)

    ..  doctest::

        >>> print(format(lilypond_file)) # doctest: +SKIP
        \version "2.19.15"
        \language "english"
        <BLANKLINE>
        #(set-global-staff-size 12)
        <BLANKLINE>
        \header {
            tagline = ##f
        }
        <BLANKLINE>
        \layout {
            indent = #0
            ragged-right = ##t
            \context {
                \Score
                \remove Bar_number_engraver
                \override SpacingSpanner.strict-grace-spacing = ##t
                \override SpacingSpanner.strict-note-spacing = ##t
                \override SpacingSpanner.uniform-stretching = ##t
                \override TupletBracket.bracket-visibility = ##t
                \override TupletBracket.minimum-length = #3
                \override TupletBracket.padding = #2
                \override TupletBracket.springs-and-rods = #ly:spanner::set-spacing-rods
                \override TupletNumber.text = #tuplet-number::calc-fraction-text
                proportionalNotationDuration = #(ly:make-moment 1 24)
                tupletFullLength = ##t
            }
        }
        <BLANKLINE>
        \paper {
            left-margin = 1\in
        }
        <BLANKLINE>
        \score {
            \new Score <<
                \new Staff {
                    c4
                    d4
                    e4
                    f4
                }
            >>
        }

    Returns LilyPond file.
    '''
    from abjad.tools import lilypondfiletools
    from abjad.tools import schemetools

    assert hasattr(music, '__illustrate__')
    lilypond_file = music.__illustrate__(**keywords)

    blocks = [_ for _ in lilypond_file.items
        if isinstance(_, lilypondfiletools.Block)
        ]
    header_block, layout_block, paper_block = None, None, None
    for block in blocks:
        if block.name == 'header':
            header_block = block
        elif block.name == 'layout':
            layout_block = block
        elif block.name == 'paper':
            paper_block = block

    # paper
    if paper_block is None:
        paper_block = lilypondfiletools.Block(name='paper')
        lilypond_file.items.insert(0, paper_block)
    paper_block.left_margin = lilypondfiletools.LilyPondDimension(1, 'in')

    # layout
    if layout_block is None:
        layout_block = lilypondfiletools.Block(name='layout')
        lilypond_file.items.insert(0, layout_block)
    # TODO: following line does nothing; must assign to paper_block instead
    #lilypond_file.line_width = lilypondfiletools.LilyPondDimension(6, 'in')
    layout_block.indent = 0
    layout_block.ragged_right = True

    # score context
    context_block = lilypondfiletools.ContextBlock(
        source_context_name='Score',
        )
    context_block.remove_commands.append('Bar_number_engraver')
    override(context_block).spacing_spanner.strict_grace_spacing = True
    override(context_block).spacing_spanner.strict_note_spacing = True
    override(context_block).spacing_spanner.uniform_stretching = True
    override(context_block).tuplet_bracket.bracket_visibility = True
    override(context_block).tuplet_bracket.padding = 2
    scheme = schemetools.Scheme('ly:spanner::set-spacing-rods')
    override(context_block).tuplet_bracket.springs_and_rods = scheme
    override(context_block).tuplet_bracket.minimum_length = 3
    scheme = schemetools.Scheme('tuplet-number::calc-fraction-text')
    override(context_block).tuplet_number.text = scheme
    moment = schemetools.SchemeMoment((1, 24))
    set_(context_block).proportionalNotationDuration = moment
    set_(context_block).tupletFullLength = True
    layout_block.items.append(context_block)

    # header
    if header_block is None:
        header_block = lilypondfiletools.Block(name='header')
        lilypond_file.items.insert(0, header_block)
    header_block.tagline = markuptools.Markup('""')

    # etc
    lilypond_file._date_time_token = None
    lilypond_file._global_staff_size = 12

    return lilypond_file



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import mathtools
from abjad.tools import systemtools
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class TimeSignature(AbjadValueObject):
    r'''Time signature.

    ..  container:: example

        **Example 1.** First time signature:

        ::

            >>> staff = Staff("c'8 d'8 e'8")
            >>> time_signature = TimeSignature((3, 8))
            >>> attach(time_signature, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \time 3/8
                c'8
                d'8
                e'8
            }

    ..  container:: example

        **Example 2.** Second time signature:

        ::

            >>> staff = Staff("c'4 d'4 e'4 f'4")
            >>> time_signature = TimeSignature((4, 4))
            >>> attach(time_signature, staff[0])
            >>> show(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff))
            \new Staff {
                \time 4/4
                c'4
                d'4
                e'4
                f'4
            }

    ..  container:: example

        **Example 3.** Scoped to the score context:

        ::

            >>> staff = Staff("c'8 d'8 e'8 c'8 d'8 e'8")
            >>> time_signature = TimeSignature((3, 8))
            >>> attach(time_signature, staff[0], scope=Score)

        Formats behind comments when no score is present:

        ::

            >>> print(format(staff))
            \new Staff {
                %%% \time 3/8 %%%
                c'8
                d'8
                e'8
                c'8
                d'8
                e'8
            }

            LilyPond defaults to common time.

        ::

            >>> show(staff) # doctest: +SKIP

        Formats normally when score is present:

        ::

            >>> score = Score([staff])
            >>> print(format(score))
            \new Score <<
                \new Staff {
                    \time 3/8
                    c'8
                    d'8
                    e'8
                    c'8
                    d'8
                    e'8
                }
            >>

        ::

            >>> show(score) # doctest: +SKIP

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_default_scope',
        '_denominator',
        '_has_non_power_of_two_denominator',
        '_multiplier',
        '_numerator',
        '_partial',
        '_partial_repr_string',
        '_suppress',
        )

    _format_slot = 'opening'

    ### INITIALIZER ###

    def __init__(self, pair=(4, 4), partial=None, suppress=None):
        from abjad.tools import scoretools
        self._default_scope = scoretools.Staff
        pair = getattr(pair, 'pair', pair)
        assert isinstance(pair, tuple), repr(pair)
        assert len(pair) == 2, repr(pair)
        numerator, denominator = pair
        assert isinstance(numerator, int), repr(numerator)
        assert isinstance(denominator, int), repr(denominator)
        self._numerator = numerator
        self._denominator = denominator
        prototype = (durationtools.Duration, type(None))
        assert isinstance(partial, prototype), repr(partial)
        self._partial = partial
        if partial is not None:
            self._partial_repr_string = ', partial=%r' % self._partial
        else:
            self._partial_repr_string = ''
        assert isinstance(suppress, (bool, type(None))), repr(suppress)
        self._suppress = suppress
        self._multiplier = self.implied_prolation
        self._has_non_power_of_two_denominator = \
            not mathtools.is_nonnegative_integer_power_of_two(
                self.denominator)

    ### SPECIAL METHODS ###

    def __add__(self, argument):
        r'''Adds time signature to `argument`.

        ..  container:: example

            **Example 1.** Adds two time signatures with the same denominator:

            >>> TimeSignature((3, 4)) + TimeSignature((3, 4))
            TimeSignature((6, 4))

        ..  container:: example

            **Example 2.** Adds two time signatures with different
            denominators:

            >>> TimeSignature((3, 4)) + TimeSignature((6, 8))
            TimeSignature((12, 8))

            Returns new time signature in terms of greatest denominator.

        Returns new time signature.
        '''
        if not isinstance(argument, type(self)):
            message = 'must be time signature: {!r}.'
            message = message.format(argument)
            raise Exception(message)
        nonreduced_1 = mathtools.NonreducedFraction(
            self.numerator,
            self.denominator,
            )
        nonreduced_2 = mathtools.NonreducedFraction(
            argument.numerator,
            argument.denominator,
            )
        result = nonreduced_1 + nonreduced_2
        result = type(self)((
            result.numerator,
            result.denominator,
            ))
        return result

    def __copy__(self, *arguments):
        r'''Copies time signature.

        Returns new time signature.
        '''
        return type(self)(
            (self.numerator, self.denominator),
            partial=self.partial,
            )

    def __eq__(self, argument):
        r'''Is true when `argument` is a time signature with numerator and
        denominator equal to this time signature. Also true when `argument` is a
        tuple with first and second elements equal to numerator and denominator
        of this time signature. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return (self.numerator == argument.numerator and
                self.denominator == argument.denominator)
        elif isinstance(argument, tuple):
            return self.numerator == argument[0] and self.denominator == argument[1]
        else:
            return False

    def __format__(self, format_specification=''):
        r'''Formats time signature.

        ..  container:: example

            **Example 1.** First time signature:

            ::

                >>> print(format(TimeSignature((3, 8))))
                indicatortools.TimeSignature((3, 8))

        ..  container:: example

            **Example 2.** Second time signature:

            ::

                >>> print(format(TimeSignature((4, 4))))
                indicatortools.TimeSignature((4, 4))

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        elif format_specification == 'lilypond':
            return self._get_lilypond_format()
        return str(self)

    def __ge__(self, argument):
        r'''Is true when duration of time signature is greater than or equal to
        duration of `argument`. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self.duration >= argument.duration
        else:
            raise TypeError

    def __gt__(self, argument):
        r'''Is true when duration of time signature is greater than duration of
        `argument`. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self.duration > argument.duration
        else:
            raise TypeError

    def __hash__(self):
        r'''Hashes time signature.

        Required to be explicitly redefined on Python 3 if __eq__ changes.

        Returns integer.
        '''
        return super(TimeSignature, self).__hash__()

    def __le__(self, argument):
        r'''Is true when duration of time signature is less than duration of
        `argument`. Otherwise false.

        Returns true or false.
        '''
        if isinstance(argument, type(self)):
            return self.duration <= argument.duration
        else:
            raise TypeError

    def __lt__(self, argument):
        r'''Is true when duration of time signature is less than duration of
        `argument`. Otherwise false.

        Returns booelan.
        '''
        if isinstance(argument, type(self)):
            return self.duration < argument.duration
        else:
            raise TypeError

    def __radd__(self, argument):
        r'''Adds `argument` to time signature.

        ..  container:: example

            **Example 1.** Adds integer to first time signature:

            >>> TimeSignature((3, 8)) + TimeSignature((4, 4))
            TimeSignature((11, 8))

        ..  container:: example

            **Example 2.** Adds integer to second time signature:

            >>> TimeSignature((4, 4)) + TimeSignature((3, 8))
            TimeSignature((11, 8))

        Returns new time signature.
        '''
        return self.__add__(argument)

    def __str__(self):
        r'''Gets string representation of time signature.

        ..  container:: example

            **Example 1.** First time signature:

            ::

                >>> str(TimeSignature((3, 8)))
                '3/8'

        ..  container:: example

            **Example 2.** Second time signature:

            ::

                >>> str(TimeSignature((4, 4)))
                '4/4'

        Returns string.
        '''
        return '{}/{}'.format(self.numerator, self.denominator)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return '{}/{}'.format(self.numerator, self.denominator)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        storage_format_is_indented = False
        if self.partial is not None or self.suppress is not None:
            storage_format_is_indented = True
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_args_values=[self.pair],
            storage_format_kwargs_names=['partial', 'suppress'],
            storage_format_is_indented=storage_format_is_indented,
            )

    def _get_lilypond_format(self):
        if self.suppress:
            return []
        elif self.partial is None:
            return r'\time {}/{}'.format(
                int(self.numerator),
                int(self.denominator),
                )
        else:
            result = []
            duration_string = self.partial.lilypond_duration_string
            partial_directive = r'\partial {}'.format(duration_string)
            result.append(partial_directive)
            string = r'\time {}/{}'.format(
                int(self.numerator),
                int(self.denominator),
                )
            result.append(string)
            return result

    ### PUBLIC PROPERTIES ###

    @property
    def default_scope(self):
        r'''Gets default scope of time signature.

        ..  container:: example

            **Example 1.** First time signature:

            ::

                >>> TimeSignature((3, 8)).default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        ..  container:: example

            **Example 2.** Second time signature:

            ::

                >>> TimeSignature((4, 4)).default_scope
                <class 'abjad.tools.scoretools.Staff.Staff'>

        Returns staff.
        '''
        return self._default_scope

    @property
    def denominator(self):
        r'''Gets denominator of time signature:

        ..  container:: example

            **Example 1.** First time signature:

            ::

                >>> TimeSignature((3, 8)).denominator
                8

        ..  container:: example

            **Example 2.** Second time signature:

            ::

                >>> TimeSignature((4, 4)).denominator
                4

        Set to positive integer.

        Returns positive integer.
        '''
        return self._denominator

    @property
    def duration(self):
        r'''Gets duration of time signature.

        ..  container:: example

            **Example 1.** First time signature:

            ::

                >>> TimeSignature((3, 8)).duration
                Duration(3, 8)

        ..  container:: example

            **Example 2.** Second time signature:

            ::

                >>> TimeSignature((4, 4)).duration
                Duration(1, 1)

        Returns duration.
        '''
        return durationtools.Duration(self.numerator, self.denominator)

    @property
    def has_non_power_of_two_denominator(self):
        r'''Is true when time signature has non-power-of-two denominator.
        Otherwise false.

        ..  container:: example

            **Example 1.** With non-power-of-two denominator:

            ::

                >>> time_signature = TimeSignature((7, 12))
                >>> time_signature.has_non_power_of_two_denominator
                True

        ..  container:: example

            **Example 2.** With power-of-two denominator:

            ::

                >>> time_signature = TimeSignature((3, 8))
                >>> time_signature.has_non_power_of_two_denominator
                False

        Returns true or false.
        '''
        return self._has_non_power_of_two_denominator

    @property
    def implied_prolation(self):
        '''Gets implied prolation of time signature.

        ..  container:: example

            **Example 1.** Implied prolation of time signature with
            power-of-two denominator:

            ::

                >>> TimeSignature((3, 8)).implied_prolation
                Multiplier(1, 1)

        ..  container:: example

            **Example 2.** Implied prolation of time signature
            with non-power-of-two denominator:

            ::

                >>> TimeSignature((7, 12)).implied_prolation
                Multiplier(2, 3)

        Returns multiplier.
        '''
        dummy_duration = durationtools.Duration(1, self.denominator)
        return dummy_duration.implied_prolation

    @property
    def numerator(self):
        r'''Gets numerator of time signature.

        ..  container:: example

            **Example 1.** First time signature:

            ::

                >>> TimeSignature((3, 8)).numerator
                3

        ..  container:: example

            **Example 2.** Second time signature:

            ::

                >>> TimeSignature((4, 4)).numerator
                4

        Set to positive integer.

        Returns positive integer.
        '''
        return self._numerator

    @property
    def pair(self):
        '''Gets numerator / denominator pair corresponding to time siganture.

        ..  container:: example

            **Example 1.** First time signature:

            ::

                >>> TimeSignature((3, 8)).pair
                (3, 8)

        ..  container:: example

            **Example 2.** Second time signature:

            ::

                >>> TimeSignature((4, 4)).pair
                (4, 4)

        Returns pair.
        '''
        return (self.numerator, self.denominator)

    @property
    def partial(self):
        r'''Gets duration of pick-up to time signature.

        ..  container:: example

            **Example 1.** First time signature:

            ::

                >>> TimeSignature((3, 8)).partial is None
                True

        ..  container:: example

            **Example 2.** Second time signature:

            ::

                >>> TimeSignature((4, 4)).partial is None
                True

        Set to duration or none.

        Defaults to none.

        Returns duration or none.
        '''
        return self._partial

    # TODO: make suppress settable only at initialization
    @property
    def suppress(self):
        r'''Is true when time signature should be suppressed in output.
        Otherwise false.

        ..  container:: example

            **Example 1.** First time signature:

            ::

                >>> TimeSignature((3, 8)).suppress is None
                True

        ..  container:: example

            **Example 2.** Second time signature:

            ::

                >>> TimeSignature((4, 4)).suppress is None
                True

        Set to boolean or none.

        Defaults to none.

        Returns boolean or none.
        '''
        return self._suppress

    @suppress.setter
    def suppress(self, argument):
        self._suppress = bool(argument)

    ### PUBLIC METHODS ###

    def with_power_of_two_denominator(
        self,
        contents_multiplier=durationtools.Multiplier(1),
        ):
        r'''Makes new time signature equivalent to current
        time signature with power-of-two denominator.

        ..  container:: example

            **Example 1.** Non-power-of-two denominator with power-of-two
            denominator:

                >>> time_signature = TimeSignature((3, 12))
                >>> time_signature.with_power_of_two_denominator()
                TimeSignature((2, 8))

        Returns new time signature.
        '''
        # check input
        contents_multiplier = durationtools.Multiplier(contents_multiplier)

        # save non_power_of_two time_signature and denominator
        non_power_of_two_denominator = self.denominator

        # find power_of_two denominator
        if contents_multiplier == durationtools.Multiplier(1):
            power_of_two_denominator = \
                mathtools.greatest_power_of_two_less_equal(
                    non_power_of_two_denominator)
        else:
            power_of_two_denominator = \
                mathtools.greatest_power_of_two_less_equal(
                    non_power_of_two_denominator, 1)

        # find power_of_two pair
        non_power_of_two_pair = mathtools.NonreducedFraction(self.pair)
        power_of_two_fraction = non_power_of_two_pair.with_denominator(
            power_of_two_denominator)
        power_of_two_pair = power_of_two_fraction.pair

        # return new power_of_two time signature
        return type(self)(power_of_two_pair)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_make_spacer_skip_measures_01():
    r'''Make list of skip-populated measures.
    '''

    divisions = [(1, 8), (5, 16), (5, 16), (1, 4)]
    measures = scoretools.make_spacer_skip_measures(divisions)
    staff = Staff(measures)

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            {
                \time 1/8
                s1 * 1/8
            }
            {
                \time 5/16
                s1 * 5/16
            }
            {
                s1 * 5/16
            }
            {
                \time 1/4
                s1 * 1/4
            }
        }
        '''
        )



# -*- coding: utf-8 -*-
import copy
from abjad.tools import durationtools
from abjad.tools import indicatortools
from abjad.tools.scoretools.Leaf import Leaf
from abjad.tools.topleveltools import detach
from abjad.tools.topleveltools import inspect_


class Note(Leaf):
    r'''A note.

    ..  container:: example

        ::

            >>> note = Note("cs''8.")
            >>> measure = Measure((3, 16), [note])
            >>> show(measure) # doctest: +SKIP

        ..  doctest::

            >>> print(format(measure))
            {
                \time 3/16
                cs''8.
            }

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Leaves'

    __slots__ = (
        '_note_head',
        )

    ### INITIALIZER ###

    def __init__(self, *arguments):
        from abjad.ly import drums
        from abjad.tools import scoretools
        from abjad.tools.topleveltools import parse
        assert len(arguments) in (0, 1, 2)
        if len(arguments) == 1 and isinstance(arguments[0], str):
            string = '{{ {} }}'.format(arguments[0])
            parsed = parse(string)
            assert len(parsed) == 1 and isinstance(parsed[0], Leaf)
            arguments = [parsed[0]]
        is_cautionary = False
        is_forced = False
        is_parenthesized = False
        if len(arguments) == 1 and isinstance(arguments[0], Leaf):
            leaf = arguments[0]
            written_pitch = None
            written_duration = leaf.written_duration
            if 'written_pitch' in dir(leaf):
                written_pitch = leaf.note_head.written_pitch
                is_cautionary = leaf.note_head.is_cautionary
                is_forced = leaf.note_head.is_forced
                is_parenthesized = leaf.note_head.is_parenthesized
            elif 'written_pitches' in dir(leaf):
                written_pitches = [x.written_pitch for x in leaf.note_heads]
                if written_pitches:
                    written_pitch = written_pitches[0]
                    is_cautionary = leaf.note_heads[0].is_cautionary
                    is_forced = leaf.note_heads[0].is_forced
                    is_parenthesized = leaf.note_heads[0].is_parenthesized
        elif len(arguments) == 2:
            written_pitch, written_duration = arguments
        elif len(arguments) == 0:
            written_pitch = 'C4'
            written_duration = durationtools.Duration(1, 4)
        else:
            message = 'can not initialize note from {!r}.'
            raise ValueError(message.format(arguments))
        Leaf.__init__(self, written_duration)
        if written_pitch is not None:
            if written_pitch not in drums:
                self.note_head = scoretools.NoteHead(
                    written_pitch=written_pitch,
                    is_cautionary=is_cautionary,
                    is_forced=is_forced,
                    is_parenthesized=is_parenthesized,
                    )
            else:
                self.note_head = scoretools.DrumNoteHead(
                    written_pitch=written_pitch,
                    is_cautionary=is_cautionary,
                    is_forced=is_forced,
                    is_parenthesized=is_parenthesized,
                    )
        else:
            self.note_head = None
        if len(arguments) == 1 and isinstance(arguments[0], Leaf):
            self._copy_override_and_set_from_leaf(arguments[0])

    ### SPECIAL METHODS ###

    def __getnewargs__(self):
        r'''Gets new arguments.

        Returns tuple.
        '''
        return (self.written_pitch, self.written_duration)

    ### PRIVATE METHODS ###

    def _divide(self, pitch=None):
        from abjad.tools import markuptools
        from abjad.tools import pitchtools
        from abjad.tools import scoretools
        pitch = pitch or pitchtools.NamedPitch('b', 3)
        pitch = pitchtools.NamedPitch(pitch)
        treble = copy.copy(self)
        bass = copy.copy(self)
        detach(markuptools.Markup, treble)
        detach(markuptools.Markup, bass)
        if treble.written_pitch < pitch:
            treble = scoretools.Rest(treble)
        if pitch <= bass.written_pitch:
            bass = scoretools.Rest(bass)
        up_markup = self._get_markup(direction=Up)
        up_markup = [copy.copy(markup) for markup in up_markup]
        down_markup = self._get_markup(direction=Down)
        down_markup = [copy.copy(markup) for markup in down_markup]
        for markup in up_markup:
            markup(treble)
        for markup in down_markup:
            markup(bass)
        return treble, bass

    def _get_sounding_pitch(self):
        from abjad.tools import instrumenttools
        from abjad.tools import pitchtools
        if 'sounding pitch' in inspect_(self).get_indicators(str):
            return self.written_pitch
        else:
            instrument = self._get_effective(instrumenttools.Instrument)
            if instrument:
                sounding_pitch = instrument.sounding_pitch_of_written_middle_c
            else:
                sounding_pitch = pitchtools.NamedPitch('C4')
            interval = pitchtools.NamedPitch('C4') - sounding_pitch
            sounding_pitch = interval.transpose(self.written_pitch)
            return sounding_pitch

    ### PRIVATE PROPERTIES ###

    @property
    def _body(self):
        result = []
        if self.note_head is not None and self.note_head.is_parenthesized:
            result.append(r'\parenthesize')
        body = ''
        if self.written_pitch:
            body += str(self.written_pitch)
            if self.note_head.is_forced:
                body += '!'
            if self.note_head.is_cautionary:
                body += '?'
        body += self._formatted_duration
        result.append(body)
        result = ['\n'.join(result)]
        return result

    @property
    def _compact_representation(self):
        return self._body[0]

    @property
    def _compact_representation_with_tie(self):
        logical_tie = self._get_logical_tie()
        if 1 < len(logical_tie) and self is not logical_tie[-1]:
            return '{} ~'.format(self._body[0])
        else:
            return self._body[0]

    ### PUBLIC PROPERTIES ###

    @property
    def note_head(self):
        r'''Gets and sets note-head of note.

        .. container:: example

            Gets note-head:

            ::

                >>> note = Note(13, (3, 16))
                >>> note.note_head
                NoteHead("cs''")

        ..  container:: example

            Sets note-head:

            ::

                >>> note = Note(13, (3, 16))
                >>> note.note_head = 14
                >>> note
                Note("d''8.")

        Returns note-head.
        '''
        return self._note_head

    @note_head.setter
    def note_head(self, arg):
        from abjad.tools.scoretools.NoteHead import NoteHead
        if isinstance(arg, type(None)):
            self._note_head = None
        elif isinstance(arg, NoteHead):
            self._note_head = arg
        else:
            note_head = NoteHead(client=self, written_pitch=arg)
            self._note_head = note_head

    @property
    def written_duration(self):
        r'''Gets and sets written duration of note.

        ..  container:: example

            Gets written duration of note.

            ::

                >>> note = Note("c'4")
                >>> note.written_duration
                Duration(1, 4)

        ..  container:: example

            Sets written duration of note:

            ::

                >>> note.written_duration = Duration(1, 16)
                >>> note.written_duration
                Duration(1, 16)

        Returns duration
        '''
        return Leaf.written_duration.fget(self)

    @written_duration.setter
    def written_duration(self, argument):
        return Leaf.written_duration.fset(self, argument)

    @property
    def written_pitch(self):
        r'''Gets and sets written pitch of note.

        ..  container:: example

            Gets written pitch of note.

            ::

                >>> note = Note(13, (3, 16))
                >>> note.written_pitch
                NamedPitch("cs''")

        ..  container:: example

            Sets written pitch of note:

            ::

                >>> note = Note(13, (3, 16))
                >>> note.written_pitch = 14
                >>> note
                Note("d''8.")

        Returns named pitch.
        '''
        if self.note_head is not None:
            if hasattr(self.note_head, 'written_pitch'):
                return self._note_head.written_pitch

    @written_pitch.setter
    def written_pitch(self, arg):
        from abjad.tools import pitchtools
        from abjad.tools.scoretools.NoteHead import NoteHead
        if arg is None:
            if self.note_head is not None:
                self.note_head.written_pitch = None
        else:
            if self.note_head is None:
                self.note_head = NoteHead(self, written_pitch=None)
            else:
                pitch = pitchtools.NamedPitch(arg)
                self.note_head.written_pitch = pitch



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_scoretools_NoteHead___setattr___01():
    r'''Slots constrain note-head attributes.
    '''

    note_head = scoretools.NoteHead("cs''")

    assert pytest.raises(AttributeError, "note_head.foo = 'bar'")



# -*- coding: utf-8 -*-
from abjad.tools import abctools


class ImageRenderSpecifier(abctools.AbjadValueObject):
    r'''An image render specifier.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Internals'

    __slots__ = (
        '_no_resize',
        '_no_stylesheet',
        '_no_trim',
        '_stylesheet',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        no_resize=None,
        no_stylesheet=None,
        no_trim=None,
        stylesheet=None,
        ):
        self._no_resize = bool(no_resize) or None
        self._no_stylesheet = bool(no_stylesheet) or None
        self._no_trim = bool(no_trim) or None
        self._stylesheet = stylesheet
        if self._no_stylesheet:
            self._stylesheet = None

    ### PUBLIC METHODS ###

    @classmethod
    def from_options(cls, **options):
        r'''Creates image specifier from `options` dictionary.

        Returns image specifier.
        '''
        no_resize = options.get('no_resize', None) or None
        no_stylesheet = options.get('no_stylesheet', None) or None
        no_trim = options.get('no_trim', None) or None
        stylesheet = options.get('stylesheet', None) or None
        if all(_ is None for _ in (
            no_resize,
            no_stylesheet,
            no_trim,
            stylesheet,
            )):
            return None
        return cls(
            no_resize=no_resize,
            no_stylesheet=no_stylesheet,
            no_trim=no_trim,
            stylesheet=stylesheet,
            )

    ### PUBLIC PROPERTIES ###

    @property
    def no_resize(self):
        r'''Is true if image should not be resized.
        Otherwise false.

        Returns true or false.
        '''
        return self._no_resize

    @property
    def no_stylesheet(self):
        r'''Is true if no stylesheet should be used with image at all.
        Otherwise false.

        Returns true or false.
        '''
        return self._no_stylesheet

    @property
    def no_trim(self):
        r'''Is true if image should not be trimmed of whitespace. Otherwise
        false.

        Returns true or false.
        '''
        return self._no_trim

    @property
    def stylesheet(self):
        r'''Gets stylesheet name to be used for image.

        Returns string or none.
        '''
        return self._stylesheet



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_RegistrationList___contains___01():
    r'''Work with mappings.
    '''

    registrations = pitchtools.RegistrationList([[('[A0, C8]', -18)]])
    assert pitchtools.Registration([('[A0, C8]', -18)]) in registrations


def test_pitchtools_RegistrationList___contains___02():
    r'''Work with mapping items.
    '''

    registrations = pitchtools.RegistrationList([[('[A0, C8]', -18)]])
    assert [('[A0, C8]', -18)] in registrations



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_PitchSet_from_selection_01():
    r'''Works with notes.
    '''

    note = Note(13, (1, 4))
    pitch_set = pitchtools.PitchSet.from_selection(note)
    assert len(pitch_set) == 1


def test_pitchtools_PitchSet_from_selection_02():
    r'''Works with chords.
    '''

    chord = Chord([13, 14, 15], (1, 4))
    pitch_set = pitchtools.PitchSet.from_selection(chord)
    assert len(pitch_set) == 3


def test_pitchtools_PitchSet_from_selection_03():
    r'''Works with chords with duplicate pitches.
    '''

    chord = Chord([13, 13, 13, 14], (1, 4))
    pitch_set = pitchtools.PitchSet.from_selection(chord)
    assert len(pitch_set) == 2


def test_pitchtools_PitchSet_from_selection_04():
    r'''Works with empty chords.
    '''

    chord = Chord([], (1, 4))
    pitch_set = pitchtools.PitchSet.from_selection(chord)
    assert len(pitch_set) == 0


def test_pitchtools_PitchSet_from_selection_05():
    r'''Works with chords.
    '''

    assert len(pitchtools.PitchSet.from_selection(
        Chord([12, 14, 18, 19], (1, 4)))) == 4



# -*- coding: utf-8 -*-
import copy
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import override
from abjad.tools.datastructuretools.TypedList import TypedList


class ClefList(TypedList):
    r'''Clef list.

    ..  container:: example

        ::

            >>> clefs = instrumenttools.ClefList(['treble', 'bass'])

        ::

            >>> f(clefs)
            instrumenttools.ClefList(
                [
                    indicatortools.Clef(
                        name='treble',
                        ),
                    indicatortools.Clef(
                        name='bass',
                        ),
                    ]
                )

        ::

            >>> 'treble' in clefs
            True

        ::

            >>> Clef('treble') in clefs
            True

        ::

            >>> 'alto' in clefs
            False

        ::

            >>> show(clefs) # doctest: +SKIP

        ..  doctest::

            >>> lilypond_file = clefs.__illustrate__()
            >>> f(lilypond_file[Staff])
            \new Staff \with {
                \override Clef.full-size-change = ##t
                \override Rest.transparent = ##t
                \override TimeSignature.stencil = ##f
            } {
                \clef "treble"
                r8
                \clef "bass"
                r8
            }

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        )

    ### SPECIAL METHODS ###

    def __illustrate__(self):
        r'''Illustrates clefs.

        ..  container:: example

            ::

                >>> show(clefs) # doctest: +SKIP

            ..  doctest::

                >>> lilypond_file = clefs.__illustrate__()
                >>> f(lilypond_file[Staff])
                \new Staff \with {
                    \override Clef.full-size-change = ##t
                    \override Rest.transparent = ##t
                    \override TimeSignature.stencil = ##f
                } {
                    \clef "treble"
                    r8
                    \clef "bass"
                    r8
                }

        Returns LilyPond file.
        '''
        import abjad
        staff = abjad.Staff()
        for clef in self:
            rest = abjad.Rest((1, 8))
            clef = copy.copy(clef)
            attach(clef, rest)
            staff.append(rest)
        override(staff).clef.full_size_change = True
        override(staff).rest.transparent = True
        override(staff).time_signature.stencil = False
        lilypond_file = abjad.LilyPondFile.new(staff)
        lilypond_file.header_block.tagline = False
        return lilypond_file

    ### PRIVATE PROPERTIES ###

    @property
    def _item_coercer(self):
        from abjad.tools import indicatortools
        return indicatortools.Clef



# -*- coding: utf-8 -*-
from abjad.tools.abctools.AbjadValueObject import AbjadValueObject


class LilyPondComment(AbjadValueObject):
    r'''LilyPond comment.

    ..  container:: example

        **Example 1.** Two-word comment:

        ::

            >>> note = Note("c'4")
            >>> comment = indicatortools.LilyPondComment('a comment')
            >>> attach(comment, note)
            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> print(format(note))
            % a comment
            c'4

    ..  container:: example

        **Example 2.** Three-word comment:

        ::

            >>> note = Note("c'4")
            >>> comment = indicatortools.LilyPondComment('yet another comment')
            >>> attach(comment, note)
            >>> show(note) # doctest: +SKIP

        ..  doctest::

            >>> print(format(note))
            % yet another comment
            c'4

    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_contents_string',
        '_default_scope',
        '_format_slot',
        )

    _format_leaf_children = False

    _allowable_format_slots = (
        'after',
        'before',
        'closing',
        'opening',
        'right',
        )

    ### INITIALIZER ###

    def __init__(self, contents_string=None, format_slot=None):
        if isinstance(contents_string, type(self)):
            argument = contents_string
            contents_string = argument.contents_string
            format_slot = format_slot or argument.format_slot
        else:
            contents_string = str(contents_string)
        self._contents_string = contents_string
        format_slot = format_slot or 'before'
        assert format_slot in self._allowable_format_slots, repr(format_slot)
        self._format_slot = format_slot

    ### SPECIAL METHODS ###

    def __str__(self):
        r'''Gets string representation of LilyPond comment.

        ..  container:: example

            **Example 1.** Two-word comment:

            ::

                >>> comment = indicatortools.LilyPondComment('a comment')
                >>> str(comment)
                '% a comment'

        ..  container:: example

            **Example 2.** Three-word comment:

            ::

                >>> comment = indicatortools.LilyPondComment('yet another comment')
                >>> str(comment)
                '% yet another comment'

        Returns string.
        '''
        return r'% {}'.format(self.contents_string)

    ### PRIVATE PROPERTIES ###

    @property
    def _contents_repr_string(self):
        return repr(self._contents_string)

    ### PRIVATE METHODS ###

    def _get_lilypond_format(self):
        return str(self)

    def _get_lilypond_format_bundle(self, component=None):
        from abjad.tools import systemtools
        lilypond_format_bundle = systemtools.LilyPondFormatBundle()
        format_slot = lilypond_format_bundle.get(self.format_slot)
        format_slot.comments.append(str(self))
        return lilypond_format_bundle

    ### PUBLIC PROPERTIES ###

    @property
    def contents_string(self):
        r'''Contents string of LilyPond comment.

        ..  container:: example

            **Example 1.** Two-word comment:

            ::

                >>> comment = indicatortools.LilyPondComment('a comment')
                >>> comment.contents_string
                'a comment'

        ..  container:: example

            **Example 2.** Three-word comment:

            ::

                >>> comment = indicatortools.LilyPondComment('yet another comment')
                >>> comment.contents_string
                'yet another comment'

        Returns string.
        '''
        return self._contents_string

    @property
    def format_slot(self):
        r'''Format slot of LilyPond comment.

        ..  container:: example

            **Example 1.** Two-word comment:

            ::

                >>> comment = indicatortools.LilyPondComment('a comment')
                >>> comment.format_slot
                'before'

        ..  container:: example

            **Example 2.** Three-word comment:

            ::

                >>> comment = indicatortools.LilyPondComment('yet another comment')
                >>> comment.format_slot
                'before'

        Defaults to ``'before'``.

        Set to allowable format slot string.

        Returns string.
        '''
        return self._format_slot

    ### PUBLIC METHODS ###

    @staticmethod
    def list_allowable_format_slots():
        r'''Lists allowable format slots.

        ..  container:: example

            **Example 1.** Default:

                >>> commands = indicatortools.LilyPondComment.list_allowable_format_slots()
                >>> for command in commands:
                ...     command
                'after'
                'before'
                'closing'
                'opening'
                'right'

        Returns tuple of strings.
        '''
        return LilyPondComment._allowable_format_slots



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_rhythmtreetools_RhythmTreeNode_parent_01():

    leaf = rhythmtreetools.RhythmTreeLeaf()
    container = rhythmtreetools.RhythmTreeContainer()
    subcontainer = rhythmtreetools.RhythmTreeContainer()

    assert leaf.parent is None
    assert container.parent is None
    assert subcontainer.parent is None

    container.append(leaf)
    assert leaf.parent is container

    container.append(subcontainer)
    assert subcontainer.parent is container
    assert leaf.parent is container
    assert container.parent is None

    subcontainer.append(leaf)
    assert leaf.parent is subcontainer
    assert subcontainer.parent is container
    assert container.parent is None

    pytest.raises(AssertionError, 'subcontainer.append(container)')



# -*- coding: utf-8 -*-
from abjad import *


def test_spannertools_OctavationSpanner___init___01():
    r'''Initialize empty octavation spanner.
    '''

    octavation = spannertools.OctavationSpanner()
    assert isinstance(octavation, spannertools.OctavationSpanner)



# -*- coding: utf-8 -*-
import os
import subprocess


def graph(
    argument,
    image_format='pdf',
    layout='dot',
    graph_attributes=None,
    node_attributes=None,
    edge_attributes=None,
    **keywords
    ):
    r'''Graphs `argument`.

    ..  container:: example

        Graphs staff:

        ::

            >>> staff = Staff("c'4 d' e' f'")
            >>> graph(staff) # doctest: +SKIP

        ..  doctest::

            >>> print(format(staff.__graph__()))
            graphtools.GraphvizGraph(
                attributes={
                    'style': 'rounded',
                    },
                children=(
                    graphtools.GraphvizNode(
                        attributes={
                            'margin': 0.05,
                            },
                        children=(
                            graphtools.GraphvizTable(
                                children=(
                                    graphtools.GraphvizTableRow(
                                        children=(
                                            graphtools.GraphvizTableCell(
                                                label='Staff',
                                                attributes={
                                                    'border': 0,
                                                    },
                                                ),
                                            ),
                                        ),
                                    ),
                                attributes={
                                    'border': 2,
                                    'cellpadding': 5,
                                    'style': 'rounded',
                                    },
                                ),
                            ),
                        name='Staff',
                        ),
                    graphtools.GraphvizSubgraph(
                        attributes={
                            'color': 'grey75',
                            'penwidth': 2,
                            },
                        children=(
                            graphtools.GraphvizNode(
                                attributes={
                                    'margin': 0.05,
                                    },
                                children=(
                                    graphtools.GraphvizTable(
                                        children=(
                                            graphtools.GraphvizTableRow(
                                                children=(
                                                    graphtools.GraphvizTableCell(
                                                        label='Note',
                                                        attributes={
                                                            'border': 0,
                                                            },
                                                        ),
                                                    ),
                                                ),
                                            graphtools.GraphvizTableHorizontalRule(),
                                            graphtools.GraphvizTableRow(
                                                children=(
                                                    graphtools.GraphvizTableCell(
                                                        label="c'4",
                                                        attributes={
                                                            'border': 0,
                                                            },
                                                        ),
                                                    ),
                                                ),
                                            ),
                                        attributes={
                                            'border': 2,
                                            'cellpadding': 5,
                                            'style': 'rounded',
                                            },
                                        ),
                                    ),
                                name='Note_0',
                                ),
                            graphtools.GraphvizNode(
                                attributes={
                                    'margin': 0.05,
                                    },
                                children=(
                                    graphtools.GraphvizTable(
                                        children=(
                                            graphtools.GraphvizTableRow(
                                                children=(
                                                    graphtools.GraphvizTableCell(
                                                        label='Note',
                                                        attributes={
                                                            'border': 0,
                                                            },
                                                        ),
                                                    ),
                                                ),
                                            graphtools.GraphvizTableHorizontalRule(),
                                            graphtools.GraphvizTableRow(
                                                children=(
                                                    graphtools.GraphvizTableCell(
                                                        label="d'4",
                                                        attributes={
                                                            'border': 0,
                                                            },
                                                        ),
                                                    ),
                                                ),
                                            ),
                                        attributes={
                                            'border': 2,
                                            'cellpadding': 5,
                                            'style': 'rounded',
                                            },
                                        ),
                                    ),
                                name='Note_1',
                                ),
                            graphtools.GraphvizNode(
                                attributes={
                                    'margin': 0.05,
                                    },
                                children=(
                                    graphtools.GraphvizTable(
                                        children=(
                                            graphtools.GraphvizTableRow(
                                                children=(
                                                    graphtools.GraphvizTableCell(
                                                        label='Note',
                                                        attributes={
                                                            'border': 0,
                                                            },
                                                        ),
                                                    ),
                                                ),
                                            graphtools.GraphvizTableHorizontalRule(),
                                            graphtools.GraphvizTableRow(
                                                children=(
                                                    graphtools.GraphvizTableCell(
                                                        label="e'4",
                                                        attributes={
                                                            'border': 0,
                                                            },
                                                        ),
                                                    ),
                                                ),
                                            ),
                                        attributes={
                                            'border': 2,
                                            'cellpadding': 5,
                                            'style': 'rounded',
                                            },
                                        ),
                                    ),
                                name='Note_2',
                                ),
                            graphtools.GraphvizNode(
                                attributes={
                                    'margin': 0.05,
                                    },
                                children=(
                                    graphtools.GraphvizTable(
                                        children=(
                                            graphtools.GraphvizTableRow(
                                                children=(
                                                    graphtools.GraphvizTableCell(
                                                        label='Note',
                                                        attributes={
                                                            'border': 0,
                                                            },
                                                        ),
                                                    ),
                                                ),
                                            graphtools.GraphvizTableHorizontalRule(),
                                            graphtools.GraphvizTableRow(
                                                children=(
                                                    graphtools.GraphvizTableCell(
                                                        label="f'4",
                                                        attributes={
                                                            'border': 0,
                                                            },
                                                        ),
                                                    ),
                                                ),
                                            ),
                                        attributes={
                                            'border': 2,
                                            'cellpadding': 5,
                                            'style': 'rounded',
                                            },
                                        ),
                                    ),
                                name='Note_3',
                                ),
                            ),
                        edge_attributes={
                            },
                        is_cluster=True,
                        name='Staff',
                        node_attributes={
                            },
                        ),
                    ),
                edge_attributes={
                    },
                is_digraph=True,
                name='G',
                node_attributes={
                    'fontname': 'Arial',
                    'shape': 'none',
                    },
                )

    ..  container:: example

        Graphs rhythm tree:

        ::

            >>> rtm_syntax = '(3 ((2 (2 1)) 2))'
            >>> rhythm_tree = rhythmtreetools.RhythmTreeParser()(rtm_syntax)[0]
            >>> topleveltools.graph(rhythm_tree) # doctest: +SKIP

        ..  doctest::

            >>> print(format(rhythm_tree.__graph__()))
            graphtools.GraphvizGraph(
                attributes={
                    'bgcolor': 'transparent',
                    'truecolor': True,
                    },
                children=(
                    graphtools.GraphvizNode(
                        attributes={
                            'label': '3',
                            'shape': 'triangle',
                            },
                        ),
                    graphtools.GraphvizNode(
                        attributes={
                            'label': '2',
                            'shape': 'triangle',
                            },
                        ),
                    graphtools.GraphvizNode(
                        attributes={
                            'label': '2',
                            'shape': 'box',
                            },
                        ),
                    graphtools.GraphvizNode(
                        attributes={
                            'label': '1',
                            'shape': 'box',
                            },
                        ),
                    graphtools.GraphvizNode(
                        attributes={
                            'label': '2',
                            'shape': 'box',
                            },
                        ),
                    ),
                edge_attributes={
                    },
                is_digraph=True,
                name='G',
                node_attributes={
                    },
                )

    Creates GraphViz object.

    Opens image in default image viewer.

    Returns none.
    '''
    from abjad import abjad_configuration
    from abjad.tools import systemtools

    if isinstance(argument, str):
        graphviz_format = argument
    else:
        assert hasattr(argument, '__graph__')
        graphviz_graph = argument.__graph__(**keywords)
        if graph_attributes:
            graphviz_graph.attributes.update(graph_attributes)
        if node_attributes:
            graphviz_graph.node_attributes.update(node_attributes)
        if edge_attributes:
            graphviz_graph.edge_attributes.update(edge_attributes)
        graphviz_format = str(graphviz_graph)

    assert image_format in ('pdf', 'png')
    valid_layouts = (
        'circo',
        'dot',
        'fdp',
        'neato',
        'osage',
        'sfdp',
        'twopi',
        )
    assert layout in valid_layouts

    message = 'cannot find `{}` command-line tool.'
    message = message.format(layout)
    message += ' Please download Graphviz from graphviz.org.'
    assert systemtools.IOManager.find_executable(layout), message

    ABJADOUTPUT = abjad_configuration['abjad_output_directory']
    systemtools.IOManager._ensure_directory_existence(ABJADOUTPUT)
    dot_path = os.path.join(
        ABJADOUTPUT,
        systemtools.IOManager.get_next_output_file_name(file_extension='dot'),
        )
    img_path = os.path.join(ABJADOUTPUT, dot_path.replace('dot', 'pdf'))

    with open(dot_path, 'w') as f:
        f.write(graphviz_format)

    command = '{} -v -T{} {} -o {}'
    command = command.format(layout, image_format, dot_path, img_path)
    subprocess.call(command, shell=True)

    pdf_viewer = abjad_configuration['pdf_viewer']
    ABJADOUTPUT = abjad_configuration['abjad_output_directory']
    systemtools.IOManager.open_file(img_path, pdf_viewer)



# -*- coding: utf-8 -*-
from abjad import *


def test_datastructuretools_TreeContainer___init___01():

    container = datastructuretools.TreeContainer()

    assert container.children == ()
    assert container.parent is None



# -*- coding: utf-8 -*-
from abjad.tools import datastructuretools
from abjad.tools.documentationtools.ReSTDirective import ReSTDirective


class ReSTTOCDirective(ReSTDirective):
    r'''A ReST TOC directive.

    ::

        >>> toc = documentationtools.ReSTTOCDirective()
        >>> for item in ['foo/index', 'bar/index', 'baz/index']:
        ...     toc.append(documentationtools.ReSTTOCItem(text=item))
        ...
        >>> toc.options['maxdepth'] = 1
        >>> toc.options['hidden'] = True
        >>> toc
        ReSTTOCDirective(
            children=(
                ReSTTOCItem(
                    text='foo/index'
                    ),
                ReSTTOCItem(
                    text='bar/index'
                    ),
                ReSTTOCItem(
                    text='baz/index'
                    ),
                ),
            directive='toctree',
            options={
                'hidden': True,
                'maxdepth': 1,
                }
            )

    ::

        >>> print(toc.rest_format)
        .. toctree::
           :hidden:
           :maxdepth: 1
        <BLANKLINE>
           foo/index
           bar/index
           baz/index

    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'reStructuredText'

    ### SPECIAL METHODS ###

    def __setitem__(self, i, argument):
        r'''Sets `i` to `argument`.

        Returns none.
        '''
        from abjad.tools import documentationtools
        newexpr = []
        for x in argument:
            if isinstance(x, str):
                newexpr.append(documentationtools.ReSTTOCItem(text=x))
            else:
                newexpr.append(x)
        datastructuretools.TreeContainer.__setitem__(self, i, newexpr)

    ### PRIVATE PROPERTIES ###

    @property
    def _children_rest_format_contributions(self):
        result = ['']
        for child in self.children:
            contribution = child._rest_format_contributions
            for x in contribution:
                if x:
                    result.append('   ' + x)
                else:
                    result.append(x)
        return result

    ### PUBLIC PROPERTIES ###

    @property
    def directive(self):
        r'''Returns ``'toctree'``.
        '''
        return 'toctree'

    @property
    def node_class(self):
        r'''Node class of ReST TOC directive.
        '''
        from abjad.tools import documentationtools
        return (
            documentationtools.ReSTTOCItem,
            )



# -*- coding: utf-8 -*-
import pytest
from abjad import *


def test_datastructuretools_TreeContainer___getitem___01():

    leaf_a = datastructuretools.TreeNode()
    leaf_b = datastructuretools.TreeNode()
    leaf_c = datastructuretools.TreeNode()

    container = datastructuretools.TreeContainer([leaf_a, leaf_b, leaf_c])

    assert container[0] is leaf_a
    assert container[1] is leaf_b
    assert container[2] is leaf_c

    pytest.raises(Exception, 'container[3]')

    assert container[-1] is leaf_c
    assert container[-2] is leaf_b
    assert container[-3] is leaf_a

    pytest.raises(Exception, 'container[-4]')



# -*- coding: utf-8 -*-
from abjad.tools.graphtools.GraphvizMixin import GraphvizMixin


class GraphvizEdge(GraphvizMixin):
    r'''A Graphviz edge.
    '''

    ### CLASS VARIABLES ###

    __documentation_section__ = 'Graphviz'

    __slots__ = (
        '_attributes',
        '_head',
        '_head_port_position',
        '_is_directed',
        '_tail',
        '_tail_port_position',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        attributes=None,
        is_directed=True,
        head_port_position=None,
        tail_port_position=None,
        ):
        GraphvizMixin.__init__(self, attributes=attributes)
        self._head = None
        self._tail = None
        self._is_directed = bool(is_directed)
        self.head_port_position = head_port_position
        self.tail_port_position = tail_port_position

    ### PRIVATE METHODS ###

    def _connect(self, tail, head):
        tail._edges.add(self)
        head._edges.add(self)
        self._tail = tail
        self._head = head

    def _disconnect(self):
        if self.tail is not None:
            self.tail._edges.remove(self)
        if self.head is not None:
            self.head._edges.remove(self)
        self._tail = None
        self._head = None

    def _get_highest_parent(self):
        from abjad.tools import graphtools
        highest_parent = None
        if isinstance(self.tail, graphtools.GraphvizField):
            tail_parentage = list(self.tail.struct.proper_parentage)
        else:
            tail_parentage = list(self.tail.proper_parentage)
        if isinstance(self.head, graphtools.GraphvizField):
            head_parentage = list(self.head.struct.proper_parentage)
        else:
            head_parentage = list(self.head.proper_parentage)
        while len(tail_parentage) and len(head_parentage) and \
            tail_parentage[-1] is head_parentage[-1]:
            highest_parent = tail_parentage[-1]
            tail_parentage.pop()
            head_parentage.pop()
        if highest_parent is None:
            message = 'highest parent can not be none.'
            raise Exception(message)
        return highest_parent

    ### PUBLIC METHODS ###

    def attach(self, tail, head):
        r'''Attaches edge from `tail` to `head`.
        '''
        from abjad.tools import graphtools
        prototype = (
            graphtools.GraphvizSubgraph,
            graphtools.GraphvizNode,
            graphtools.GraphvizField,
            )
        assert isinstance(tail, prototype)
        assert isinstance(head, prototype)
        self._disconnect()
        self._connect(tail, head)

    def detach(self):
        r'''Detaches edge.
        '''
        self._disconnect()

    ### PRIVATE PROPERTIES ###

    @property
    def _graphviz_format_contributions(self):
        connection = '->'
        if not self.is_directed:
            connection = '--'
        tail_name = self.tail.canonical_name
        if self._tail_port_position is not None:
            tail_name = '{}:{}'.format(tail_name, self._tail_port_position)
        head_name = self.head.canonical_name
        if self._head_port_position is not None:
            head_name = '{}:{}'.format(head_name, self._head_port_position)
        edge_def = '{} {} {}'.format(
            self._format_value(tail_name),
            connection,
            self._format_value(head_name),
            )
        if len(self.attributes):
            result = self._format_attribute_list(self.attributes)
            result[0] = '{} {}'.format(edge_def, result[0])
            return result
        return [edge_def + ';']

    ### PUBLIC PROPERTIES ###

    @property
    def head(self):
        r'''Head of Graphviz edge.
        '''
        return self._head

    @property
    def head_port_position(self):
        r'''Head port position.
        '''
        return self._head_port_position

    @head_port_position.setter
    def head_port_position(self, argument):
        self._head_port_position = argument

    @property
    def is_directed(self):
        r'''Is true when Graphviz edge is directed. Otherwise false.

        Returns true or false.
        '''
        return self._is_directed

    @is_directed.setter
    def is_directed(self, arg):
        self._is_directed = bool(arg)

    @property
    def tail(self):
        r'''Tail of Graphviz edge.
        '''
        return self._tail

    @property
    def tail_port_position(self):
        r'''Tail port position.
        '''
        return self._tail_port_position

    @tail_port_position.setter
    def tail_port_position(self, argument):
        self._tail_port_position = argument



# -*- coding: utf-8 -*-
import sys
from abjad import *


def test_spannertools_Spanner__apply_overrides_01():

    overrides = {
        'text_spanner__dash_period': '1.5',
        'text_spanner__bound_details__right_broken__padding': '0',
        'text_spanner__bound_details__left_broken__text': "markuptools.Markup((markuptools.MarkupCommand('italic','(fl.)'),markuptools.MarkupCommand('hspace',1)))",
        'text_spanner__bound_details__left__stencil_align_dir_y': '0',
        'text_spanner__bound_details__right__text': "markuptools.Markup((markuptools.MarkupCommand('draw-line',schemetools.SchemePair(0, -1)),))",
        'text_spanner__bound_details__right_broken__text': 'None',
        'text_spanner__bound_details__right__padding': '1',
        'text_spanner__bound_details__left__padding': '-1',
        'text_spanner__dash_fraction': '0.25',
    }

    if sys.version_info[0] == 2:
        overrides['text_spanner__bound_details__left__text'] = \
            "markuptools.Markup((markuptools.MarkupCommand('italic','\\xe2\\x80\\x9cwhite\\xe2\\x80\\x9d flautando'),markuptools.MarkupCommand('hspace',1)))"
    else:
        overrides['text_spanner__bound_details__left__text'] = \
            "markuptools.Markup((markuptools.MarkupCommand('italic','“white” flautando'),markuptools.MarkupCommand('hspace',1)))"

    white_flautando_spanner = spannertools.TextSpanner(overrides=overrides)

    staff = Staff("c'4 d'4 e'4 f'4")
    attach(white_flautando_spanner, staff[:])

    assert format(staff) == stringtools.normalize(
        r'''
        \new Staff {
            \override TextSpanner.bound-details.left-broken.text = \markup {
                \italic
                    (fl.)
                \hspace
                    #1
                }
            \override TextSpanner.bound-details.left.padding = #'-1
            \override TextSpanner.bound-details.left.stencil-align-dir-y = #'0
            \override TextSpanner.bound-details.left.text = \markup {
                \italic
                    "“white” flautando"
                \hspace
                    #1
                }
            \override TextSpanner.bound-details.right-broken.padding = #'0
            \override TextSpanner.bound-details.right-broken.text = #'None
            \override TextSpanner.bound-details.right.padding = #'1
            \override TextSpanner.bound-details.right.text = \markup {
                \draw-line
                    #'(0 . -1)
                }
            \override TextSpanner.dash-fraction = #'0.25
            \override TextSpanner.dash-period = #'1.5
            c'4 \startTextSpan
            d'4
            e'4
            f'4 \stopTextSpan
            \revert TextSpanner.bound-details
            \revert TextSpanner.dash-fraction
            \revert TextSpanner.dash-period
        }
        '''
        ), format(staff)



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_extend_measures_and_apply_full_measure_tuplets_to_contents_of_measures_01():
    r'''Tupletize one measure, supplement one note.
    '''

    measure = Measure((4, 8), scoretools.make_repeated_notes(4))
    scoretools.extend_measures_and_apply_full_measure_tuplets(
        measure, scoretools.make_repeated_notes(1))

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 4/8
            \times 4/5 {
                c'8
                c'8
                c'8
                c'8
                c'8
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()


def test_scoretools_extend_measures_and_apply_full_measure_tuplets_to_contents_of_measures_02():
    r'''Tupletize one measure, supplement one rest.
    '''

    measure = Measure((4, 8), "c'8 c'8 c'8 c'8")
    scoretools.extend_measures_and_apply_full_measure_tuplets(
        measure, [Rest((1, 4))])

    assert format(measure) == stringtools.normalize(
        r'''
        {
            \time 4/8
            \times 2/3 {
                c'8
                c'8
                c'8
                c'8
                r4
            }
        }
        '''
        )

    assert inspect_(measure).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_scoretools_NoteHeadList___setitem___01():
    r'''Set note-head with pitch number.
    '''

    chord = Chord("<c' d'>4")
    chord.note_heads[1] = 4

    assert format(chord) == "<c' e'>4"


def test_scoretools_NoteHeadList___setitem___02():
    '''Set note-head with pitch.
    '''

    chord = Chord("<c' d'>4")
    chord.note_heads[1] = NamedPitch("e'")

    assert format(chord) == "<c' e'>4"


def test_scoretools_NoteHeadList___setitem___03():
    r'''Set note-head with tweaked note-head.
    '''

    chord = Chord("<c' cs'' f''>4")
    note_head = scoretools.NoteHead(3)
    note_head.tweak.color = 'red'
    chord.note_heads[0] = note_head

    assert format(chord) == stringtools.normalize(
        r'''
        <
            \tweak color #red
            ef'
            cs''
            f''
        >4
        '''
        )

    assert inspect_(chord).is_well_formed()



# -*- coding: utf-8 -*-
from abjad import *


def test_indicatortools_Clef_middle_c_position_01():

    assert Clef('treble').middle_c_position == pitchtools.StaffPosition(-6)
    assert Clef('alto').middle_c_position == pitchtools.StaffPosition(0)
    assert Clef('tenor').middle_c_position == pitchtools.StaffPosition(2)
    assert Clef('bass').middle_c_position == pitchtools.StaffPosition(6)
    assert Clef('treble^8').middle_c_position == pitchtools.StaffPosition(-13)
    assert Clef('alto^15').middle_c_position == pitchtools.StaffPosition(-13)
    assert Clef('tenor_8').middle_c_position == pitchtools.StaffPosition(9)
    assert Clef('bass_15').middle_c_position == pitchtools.StaffPosition(19)



# -*- coding: utf-8 -*-
from abjad import *


def test_tonalanalysistools_TonalAnalysisAgent_analyze_incomplete_tonal_functions_01():

    chord = Chord("<g' b'>4")
    key_signature = KeySignature('c', 'major')
    tonal_function = tonalanalysistools.RomanNumeral('V')
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_incomplete_tonal_functions(key_signature)
    assert result == [tonal_function]

    chord = Chord("<g' bf'>4")
    tonal_function = tonalanalysistools.RomanNumeral('v')
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_incomplete_tonal_functions(key_signature)
    assert result == [tonal_function]


def test_tonalanalysistools_TonalAnalysisAgent_analyze_incomplete_tonal_functions_02():

    key_signature = KeySignature('c', 'major')
    chord = Chord("<f g b>4")
    tonal_function = tonalanalysistools.RomanNumeral('V4/3')
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_incomplete_tonal_functions(key_signature)
    assert result == [tonal_function]

    chord = Chord("<fs g b>4")
    tonal_function = tonalanalysistools.RomanNumeral('VM4/3')
    selection = tonalanalysistools.select(chord)
    result = selection.analyze_incomplete_tonal_functions(key_signature)
    assert result == [tonal_function]



# -*- coding: utf-8 -*-
from abjad.tools import schemetools
from abjad.tools.topleveltools import override
from abjad.tools.topleveltools import set_


def configure_score(score):
    r'''Configures score.
    '''

    spacing_vector = schemetools.make_spacing_vector(0, 0, 8, 0)
    override(score).vertical_axis_group.staff_staff_spacing = spacing_vector
    override(score).staff_grouper.staff_staff_spacing = spacing_vector
    override(score).staff_symbol.thickness = 0.5
    set_(score).mark_formatter = schemetools.Scheme('format-mark-box-numbers')


# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedInversionEquivalentIntervalClass__init_by_self_reference_01():

    dic_1 = pitchtools.NamedInversionEquivalentIntervalClass('major', 2)
    dic_2 = pitchtools.NamedInversionEquivalentIntervalClass(dic_1)

    assert str(dic_1) == '+M2'
    assert str(dic_2) == '+M2'



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import markuptools
from abjad.tools import scoretools
from abjad.tools.topleveltools import attach
from abjad.tools.topleveltools import iterate
from abjad.tools.topleveltools import mutate


def apply_expressive_marks(score):
    r'''Applies expressive marks to score.
    '''

    voice = score['First Violin Voice']
    markup = markuptools.Markup(
        r'\left-column { div. \line { con sord. } }', Up)
    attach(markup, voice[6][1])
    markup = markuptools.Markup('sim.', Up)
    attach(markup, voice[8][0])
    markup = markuptools.Markup('uniti', Up)
    attach(markup, voice[58][3])
    markup = markuptools.Markup('div.', Up)
    attach(markup, voice[59][0])
    markup = markuptools.Markup('uniti', Up)
    attach(markup, voice[63][3])

    voice = score['Second Violin Voice']
    markup = markuptools.Markup('div.', Up)
    attach(markup, voice[7][0])
    markup = markuptools.Markup('uniti', Up)
    attach(markup, voice[66][1])
    markup = markuptools.Markup('div.', Up)
    attach(markup, voice[67][0])
    markup = markuptools.Markup('uniti', Up)
    attach(markup, voice[74][0])

    voice = score['Viola Voice']
    markup = markuptools.Markup('sole', Up)
    attach(markup, voice[8][0])

    voice = score['Cello Voice']
    markup = markuptools.Markup('div.', Up)
    attach(markup, voice[10][0])
    markup = markuptools.Markup('uniti', Up)
    attach(markup, voice[74][0])
    markup = markuptools.Markup('uniti', Up)
    attach(markup, voice[84][1])
    markup = markuptools.Markup(r'\italic { espr. }', Down)
    attach(markup, voice[86][0])
    markup = markuptools.Markup(r'\italic { molto espr. }', Down)
    attach(markup, voice[88][1])

    voice = score['Bass Voice']
    markup = markuptools.Markup('div.', Up)
    attach(markup, voice[14][0])
    markup = markuptools.Markup(r'\italic { espr. }', Down)
    attach(markup, voice[86][0])
    mutate(voice[88][:]).split(
        [durationtools.Duration(1, 1), durationtools.Duration(1, 2)]
        )
    markup = markuptools.Markup(r'\italic { molto espr. }', Down)
    attach(markup, voice[88][1])
    markup = markuptools.Markup('uniti', Up)
    attach(markup, voice[99][1])

    strings_staff_group = score['Strings Staff Group']
    for voice in iterate(strings_staff_group).by_class(scoretools.Voice):
        markup = markuptools.Markup(r'\italic { (non dim.) }', Down)
        attach(markup, voice[102][0])


# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NamedPitch__spell_pitch_number_01():

    accidental = NamedPitch._spell_pitch_number(12, 'b')
    assert accidental == (pitchtools.Accidental('s'), 4)
    accidental = NamedPitch._spell_pitch_number(12, 'c')
    assert accidental == (pitchtools.Accidental(''), 5)
    accidental = NamedPitch._spell_pitch_number(12, 'd')
    assert accidental == (pitchtools.Accidental('ff'), 5)


def test_pitchtools_NamedPitch__spell_pitch_number_02():

    accidental = NamedPitch._spell_pitch_number(13, 'b')
    assert accidental == (pitchtools.Accidental('ss'), 4)
    accidental = NamedPitch._spell_pitch_number(13, 'c')
    assert accidental == (pitchtools.Accidental('s'), 5)
    accidental = NamedPitch._spell_pitch_number(13, 'd')
    assert accidental == (pitchtools.Accidental('f'), 5)


def test_pitchtools_NamedPitch__spell_pitch_number_03():

    t = NamedPitch._spell_pitch_number(14, 'c')
    assert t == (pitchtools.Accidental('ss'), 5)
    t = NamedPitch._spell_pitch_number(14, 'd')
    assert t == (pitchtools.Accidental(''), 5)
    t = NamedPitch._spell_pitch_number(14, 'e')
    assert t == (pitchtools.Accidental('ff'), 5)



# -*- coding: utf-8 -*-
import pytest
from abjad import *
from abjad.tools.lilypondparsertools import LilyPondParser


def test_lilypondparsertools_LilyPondParser__indicators__Articulation_01():

    target = Staff(scoretools.make_notes(["c''"], [(1, 4)] * 6 + [(1, 2)]))
    articulation = Articulation('marcato', Up)
    attach(articulation, target[0])
    articulation = Articulation('stopped', Down)
    attach(articulation, target[1])
    articulation = Articulation('tenuto')
    attach(articulation, target[2])
    articulation = Articulation('staccatissimo')
    attach(articulation, target[3])
    articulation = Articulation('accent')
    attach(articulation, target[4])
    articulation = Articulation('staccato')
    attach(articulation, target[5])
    articulation = Articulation('portato')
    attach(articulation, target[6])

    assert format(target) == stringtools.normalize(
        r'''
        \new Staff {
            c''4 ^\marcato
            c''4 _\stopped
            c''4 -\tenuto
            c''4 -\staccatissimo
            c''4 -\accent
            c''4 -\staccato
            c''2 -\portato
        }
        '''
        )

    string = r'''\new Staff { c''4^^ c''_+ c''-- c''-| c''4-> c''-. c''2-_ }'''

    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and \
        target is not result
    for x in result:
        assert 1 == len(inspect_(x).get_indicators(Articulation))


def test_lilypondparsertools_LilyPondParser__indicators__Articulation_02():

    target = Staff([Note("c'", (1, 4))])
    articulation = Articulation('marcato', Up)
    attach(articulation, target[0])
    articulation = Articulation('stopped', Down)
    attach(articulation, target[0])
    articulation = Articulation('tenuto')
    attach(articulation, target[0])
    articulation = Articulation('staccatissimo')
    attach(articulation, target[0])
    articulation = Articulation('accent')
    attach(articulation, target[0])
    articulation = Articulation('staccato')
    attach(articulation, target[0])
    articulation = Articulation('portato')
    attach(articulation, target[0])

    r'''
    \new Staff {
        c'4 ^\marcato _\stopped -\tenuto -\staccatissimo -\accent -\staccato -\portato
    }
    '''

    string = r'''\new Staff { c'4 ^^ _+ -- -| -> -. -_ }'''


    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and \
        target is not result
    assert 7 == len(inspect_(result[0]).get_indicators(Articulation))


def test_lilypondparsertools_LilyPondParser__indicators__Articulation_03():

    target = Container(scoretools.make_notes(
        ["c''", "c''", "b'", "c''"],
        [(1, 4), (1, 4), (1, 2), (1, 1)]))

    articulation = Articulation('staccato')
    attach(articulation, target[0])
    articulation = Articulation('mordent')
    attach(articulation, target[1])
    articulation = Articulation('turn')
    attach(articulation, target[2])
    articulation = Articulation('fermata')
    attach(articulation, target[3])

    assert format(target) == stringtools.normalize(
        r'''
        {
            c''4 -\staccato
            c''4 -\mordent
            b'2 -\turn
            c''1 -\fermata
        }
        '''
        )

    string = r'''{ c''4\staccato c''\mordent b'2\turn c''1\fermata }'''

    parser = LilyPondParser()
    result = parser(string)
    assert format(target) == format(result) and \
        target is not result
    for x in result:
        assert 1 == len(inspect_(x).get_indicators(Articulation))



# -*- coding: utf-8 -*-
from abjad.tools import durationtools
from abjad.tools import graphtools
from abjad.tools import indicatortools
from abjad.tools import mathtools
from abjad.tools import rhythmtreetools
from abjad.tools import scoretools
from abjad.tools import sequencetools
from abjad.tools import systemtools
from abjad.tools.abctools import AbjadObject


class Meter(AbjadObject):
    '''A meter.

    Meter models a common practice understanding of beats and other levels of
    rhythmic organization structured as a tree. Meter structure corresponds to
    the monotonically increasing sequence of factors in the numerator of a
    given time signature. Successively deeper levels of the tree divide time by
    successive factors.

    ..  container:: example

        **Example 1.** Duple meter:

        ::

            >>> meter = metertools.Meter((2, 4))
            >>> meter
            Meter('(2/4 (1/4 1/4))')
            >>> print(meter.pretty_rtm_format)
            (2/4 (
                1/4
                1/4))

        ::

            >>> graph(meter) # doctest: +SKIP

        `2/4` comprises two beats.

    ..  container:: example

        **Example 2.** Triple meter:

        ::

            >>> meter = metertools.Meter((3, 4))
            >>> print(meter.pretty_rtm_format)
            (3/4 (
                1/4
                1/4
                1/4))

        ::

            >>> graph(meter) # doctest: +SKIP

        `3/4` comprises three beats.

    ..  container:: example

        **Example 3.** Quadruple meter:

        ::

            >>> meter = metertools.Meter((4, 4))
            >>> meter
            Meter('(4/4 (1/4 1/4 1/4 1/4))')
            >>> print(meter.pretty_rtm_format)
            (4/4 (
                1/4
                1/4
                1/4
                1/4))

        ::

            >>> graph(meter) # doctest: +SKIP

        `4/4` comprises four beats.

    ..  container:: example

        **Example 3.** Compound triple meter:

        ::

            >>> meter = metertools.Meter((6, 8))
            >>> print(meter.pretty_rtm_format)
            (6/8 (
                (3/8 (
                    1/8
                    1/8
                    1/8))
                (3/8 (
                    1/8
                    1/8
                    1/8))))

        ::

            >>> graph(meter) # doctest: +SKIP

        `6/8` comprises two beats of three parts each.

    ..  container:: example

        **Example 4.** Another compound triple meter:

        ::

            >>> meter = metertools.Meter((12, 8))
            >>> print(meter.pretty_rtm_format)
            (12/8 (
                (3/8 (
                    1/8
                    1/8
                    1/8))
                (3/8 (
                    1/8
                    1/8
                    1/8))
                (3/8 (
                    1/8
                    1/8
                    1/8))
                (3/8 (
                    1/8
                    1/8
                    1/8))))

        ::

            >>> graph(meter) # doctest: +SKIP

        `12/8` comprises four beats of three parts each.

    ..  container:: example

        **Example 5.** An asymmetric meter:

        ::

            >>> meter = metertools.Meter((5, 4))
            >>> print(meter.pretty_rtm_format)
            (5/4 (
                (3/4 (
                    1/4
                    1/4
                    1/4))
                (2/4 (
                    1/4
                    1/4))))

        ::

            >>> graph(meter) # doctest: +SKIP

        `5/4` comprises two unequal beats. By default unequal beats
        are arranged from greatest to least.

    ..  container:: example

        **Example 6.** Another asymmetric meter:

        ::

            >>> meter = metertools.Meter((7, 4))
            >>> print(meter.pretty_rtm_format)
            (7/4 (
                (3/4 (
                    1/4
                    1/4
                    1/4))
                (2/4 (
                    1/4
                    1/4))
                (2/4 (
                    1/4
                    1/4))))

        ::

            >>> graph(meter) # doctest: +SKIP

        `7/4` comprises three unequal beats. Beats are arranged from
        greatest to least by default.

    ..  container:: example

        **Example 7.** The same asymmetric meter structured differently:

        ::

            >>> meter = metertools.Meter(
            ...     (7, 4),
            ...     decrease_durations_monotonically=False,
            ...     )
            >>> print(meter.pretty_rtm_format)
            (7/4 (
                (2/4 (
                    1/4
                    1/4))
                (2/4 (
                    1/4
                    1/4))
                (3/4 (
                    1/4
                    1/4
                    1/4))))

        ::

            >>> graph(meter) # doctest: +SKIP

        `7/4` with beats arragned from least to greatest.

    ..  container:: example

        **Example 8a.** Meter interpreted by default as containing two compound
        beats:

        ::

            >>> meter = metertools.Meter((6, 4))
            >>> meter
            Meter('(6/4 ((3/4 (1/4 1/4 1/4)) (3/4 (1/4 1/4 1/4))))')
            >>> print(meter.pretty_rtm_format)
            (6/4 (
                (3/4 (
                    1/4
                    1/4
                    1/4))
                (3/4 (
                    1/4
                    1/4
                    1/4))))

        ::

            >>> graph(meter) # doctest: +SKIP

        **Example 8b.** Same meter customized to contain four compound beats:

        ::

            >>> parser = rhythmtreetools.RhythmTreeParser()
            >>> meter = metertools.Meter('(6/4 ((3/8 (1/8 1/8 1/8)) (3/8 (1/8 1/8 1/8)) (3/8 (1/8 1/8 1/8)) (3/8 (1/8 1/8 1/8))))')
            >>> meter
            Meter('(6/4 ((3/8 (1/8 1/8 1/8)) (3/8 (1/8 1/8 1/8)) (3/8 (1/8 1/8 1/8)) (3/8 (1/8 1/8 1/8))))')
            >>> print(meter.pretty_rtm_format)
            (6/4 (
                (3/8 (
                    1/8
                    1/8
                    1/8))
                (3/8 (
                    1/8
                    1/8
                    1/8))
                (3/8 (
                    1/8
                    1/8
                    1/8))
                (3/8 (
                    1/8
                    1/8
                    1/8))))

        ::

            >>> graph(meter) # doctest: +SKIP

    Prime divisions greater than ``3`` are converted to sequences of ``2``
    and ``3`` summing to that prime. Summands are arranged from greatest
    to least by default. This means that ``5`` becomes ``3+2`` and ``7``
    becomes ``3+2+2`` in the examples above.
    '''

    ### CLASS VARIABLES ###

    __slots__ = (
        '_decrease_durations_monotonically',
        '_denominator',
        '_numerator',
        '_preferred_boundary_depth',
        '_root_node',
        )

    ### INITIALIZER ###

    def __init__(
        self,
        arg=None,
        decrease_durations_monotonically=True,
        preferred_boundary_depth=None,
        ):

        arg = arg or (4, 4)
        assert isinstance(preferred_boundary_depth, (int, type(None)))
        self._preferred_boundary_depth = preferred_boundary_depth

        def recurse(
            node,
            factors,
            denominator,
            decrease_durations_monotonically,
            ):
            if factors:
                factor, factors = factors[0], factors[1:]
                preprolated_duration = \
                    node.preprolated_duration.__div__(factor)
                #if factor in (2, 3, 4, 5):
                if factor in (2, 3, 4):
                    if factors:
                        for _ in range(factor):
                            child = rhythmtreetools.RhythmTreeContainer(
                                preprolated_duration=preprolated_duration)
                            node.append(child)
                            recurse(
                                child,
                                factors,
                                denominator,
                                decrease_durations_monotonically,
                                )
                    else:
                        for _ in range(factor):
                            node.append(
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=(1, denominator)))
                else:
                    parts = [3]
                    total = 3
                    while total < factor:
                        if decrease_durations_monotonically:
                            parts.append(2)
                        else:
                            parts.insert(0, 2)
                        total += 2
                    for part in parts:
                        grouping = rhythmtreetools.RhythmTreeContainer(
                            preprolated_duration=part * preprolated_duration)
                        if factors:
                            for _ in range(part):
                                child = rhythmtreetools.RhythmTreeContainer(
                                    preprolated_duration=preprolated_duration)
                                grouping.append(child)
                                recurse(
                                    child,
                                    factors,
                                    denominator,
                                    decrease_durations_monotonically,
                                    )
                        else:
                            for _ in range(part):
                                grouping.append(
                                    rhythmtreetools.RhythmTreeLeaf(
                                        preprolated_duration=(1, denominator)))
                        node.append(grouping)
            else:
                node.extend([rhythmtreetools.RhythmTreeLeaf(
                    preprolated_duration=(1, denominator))
                    for _ in range(node.preprolated_duration.numerator)])

        decrease_durations_monotonically = \
            bool(decrease_durations_monotonically)

        if isinstance(arg, type(self)):
            root = arg.root_node
            numerator, denominator = arg.numerator, arg.denominator
            decrease_durations_monotonically = \
                arg.decrease_durations_monotonically

        elif isinstance(arg, (str, rhythmtreetools.RhythmTreeContainer)):
            if isinstance(arg, str):
                parsed = rhythmtreetools.RhythmTreeParser()(arg)
                assert len(parsed) == 1
                root = parsed[0]
            else:
                root = arg
            for node in root.nodes:
                assert node.prolation == 1
            numerator = root.preprolated_duration.numerator
            denominator = root.preprolated_duration.denominator

        elif (isinstance(arg, (tuple, scoretools.Measure)) or
            (hasattr(arg, 'numerator') and hasattr(arg, 'denominator'))):
            if isinstance(arg, tuple):
                fraction = mathtools.NonreducedFraction(arg)
            elif isinstance(arg, scoretools.Measure):
                time_signature = arg._get_effective(
                    indicatortools.TimeSignature)
                fraction = mathtools.NonreducedFraction(
                    time_signature.numerator, time_signature.denominator)
            else:
                fraction = mathtools.NonreducedFraction(
                    arg.numerator, arg.denominator)
            numerator, denominator = fraction.numerator, fraction.denominator
            factors = mathtools.factors(numerator)
            # group two nested levels of 2s into a 4
            if 1 < len(factors) and factors[0] == factors[1] == 2:
                factors[0:2] = [4]
            root = rhythmtreetools.RhythmTreeContainer(
                preprolated_duration=fraction)
            recurse(
                root,
                factors,
                denominator,
                decrease_durations_monotonically,
                )

        else:
            message = 'can not initialize {}: {!r}.'
            message = message.format(type(self).__name__, arg)
            raise ValueError(message)

        self._root_node = root
        self._numerator = numerator
        self._denominator = denominator
        self._decrease_durations_monotonically = \
            decrease_durations_monotonically

    ### SPECIAL METHODS ###

    def __eq__(self, argument):
        r'''Is true when `argument` is a meter with an rtm format equal to that of
        this meter. Otherwise false.

        Returns true or false.
        '''
        if type(self) == type(argument):
            if self.rtm_format == argument.rtm_format:
                return True
        return False

    def __format__(self, format_specification=''):
        r'''Formats meter.

        ..  container:: example

            **Example 1.** Gets storage format of ``7/4``:

            ::

                >>> meter = metertools.Meter((7, 4))
                >>> print(format(meter))
                metertools.Meter(
                    '(7/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)) (2/4 (1/4 1/4))))'
                    )

        Set `format_specification` to `''` or `'storage'`.
        Interprets `''` equal to `'storage'`.

        Returns string.
        '''
        from abjad.tools import systemtools
        if format_specification in ('', 'storage'):
            return systemtools.StorageFormatAgent(self).get_storage_format()
        return str(self)

    def __graph__(self, **keywords):
        r'''Gets Graphviz format of meter.

        ..  container:: example

            **Example 1.** Graphs ``7/4``:

            ::

                >>> meter = metertools.Meter((7, 4))
                >>> meter_graph = meter.__graph__()
                >>> graph(meter_graph) # doctest: +SKIP

            ..  doctest::

                >>> print(str(meter_graph))
                digraph G {
                    graph [bgcolor=transparent,
                        fontname=Arial,
                        penwidth=2,
                        truecolor=true];
                    node [fontname=Arial,
                        fontsize=12,
                        penwidth=2];
                    edge [penwidth=2];
                    node_0 [label="7/4",
                        shape=triangle];
                    node_1 [label="3/4",
                        shape=triangle];
                    node_2 [label="1/4",
                        shape=box];
                    node_3 [label="1/4",
                        shape=box];
                    node_4 [label="1/4",
                        shape=box];
                    node_5 [label="2/4",
                        shape=triangle];
                    node_6 [label="1/4",
                        shape=box];
                    node_7 [label="1/4",
                        shape=box];
                    node_8 [label="2/4",
                        shape=triangle];
                    node_9 [label="1/4",
                        shape=box];
                    node_10 [label="1/4",
                        shape=box];
                    subgraph cluster_cluster_offsets {
                        graph [style=rounded];
                        node_11_0 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 0 | <f_0_1> +++ }",
                            shape=Mrecord,
                            style=filled];
                        node_11_1 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 1/4 | <f_0_1> + }",
                            shape=Mrecord,
                            style=filled];
                        node_11_2 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 1/2 | <f_0_1> + }",
                            shape=Mrecord,
                            style=filled];
                        node_11_3 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 3/4 | <f_0_1> ++ }",
                            shape=Mrecord,
                            style=filled];
                        node_11_4 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 1 | <f_0_1> + }",
                            shape=Mrecord,
                            style=filled];
                        node_11_5 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 5/4 | <f_0_1> ++ }",
                            shape=Mrecord,
                            style=filled];
                        node_11_6 [color=white,
                            fillcolor=black,
                            fontcolor=white,
                            fontname="Arial bold",
                            label="{ <f_0_0> 3/2 | <f_0_1> + }",
                            shape=Mrecord,
                            style=filled];
                        node_11_7 [label="{ <f_0_0> 7/4 | <f_0_1> +++ }",
                            shape=Mrecord];
                    }
                    node_0 -> node_1;
                    node_0 -> node_5;
                    node_0 -> node_8;
                    node_1 -> node_2;
                    node_1 -> node_3;
                    node_1 -> node_4;
                    node_2 -> node_11_0 [style=dotted];
                    node_2 -> node_11_1 [style=dotted];
                    node_3 -> node_11_1 [style=dotted];
                    node_3 -> node_11_2 [style=dotted];
                    node_4 -> node_11_2 [style=dotted];
                    node_4 -> node_11_3 [style=dotted];
                    node_5 -> node_6;
                    node_5 -> node_7;
                    node_6 -> node_11_3 [style=dotted];
                    node_6 -> node_11_4 [style=dotted];
                    node_7 -> node_11_4 [style=dotted];
                    node_7 -> node_11_5 [style=dotted];
                    node_8 -> node_9;
                    node_8 -> node_10;
                    node_9 -> node_11_5 [style=dotted];
                    node_9 -> node_11_6 [style=dotted];
                    node_10 -> node_11_6 [style=dotted];
                    node_10 -> node_11_7 [style=dotted];
                }

        Returns Graphviz graph.
        '''
        def make_offset_node(
            offset,
            leaf_one=None,
            leaf_two=None,
            is_last=False,
            ):
            if not is_last:
                offset_node = graphtools.GraphvizNode(
                    attributes={
                        'shape': 'Mrecord',
                        'style': 'filled',
                        'color': 'white',
                        'fontname': 'Arial bold',
                        'fontcolor': 'white',
                        'fillcolor': 'black',
                        },
                    )
            else:
                offset_node = graphtools.GraphvizNode(
                    attributes={
                        'shape': 'Mrecord',
                        },
                    )
            offset_field = graphtools.GraphvizField(
                label=str(offset),
                )
            weight_field = graphtools.GraphvizField(
                label='+' * offsets[offset],
                )
            group = graphtools.GraphvizGroup()
            group.extend([offset_field, weight_field])
            offset_node.append(group)
            offset_subgraph.append(offset_node)
            leaf_one_node = node_mapping[leaf_one]
            edge = graphtools.GraphvizEdge(
                attributes={'style': 'dotted'},
                )
            edge.attach(leaf_one_node, offset_node)
            if leaf_two:
                leaf_two_node = node_mapping[leaf_two]
                edge = graphtools.GraphvizEdge(
                    attributes={'style': 'dotted'},
                    )
                edge.attach(leaf_two_node, offset_node)
        from abjad.tools import metertools
        offsets = metertools.MetricAccentKernel.count_offsets(
            sequencetools.Sequence(self.depthwise_offset_inventory).flatten())
        graph = graphtools.GraphvizGraph(
            name='G',
            attributes={
                'bgcolor': 'transparent',
                'fontname': 'Arial',
                'penwidth': 2,
                'truecolor': True,
                },
            edge_attributes={
                'penwidth': 2,
                },
            node_attributes={
                'fontname': 'Arial',
                'fontsize': 12,
                'penwidth': 2,
                },
            )
        node_mapping = {}
        for node in self._root_node.nodes:
            graphviz_node = graphtools.GraphvizNode()
            graphviz_node.attributes['label'] = str(node.preprolated_duration)
            if isinstance(node, rhythmtreetools.RhythmTreeContainer):
                graphviz_node.attributes['shape'] = 'triangle'
            else:
                graphviz_node.attributes['shape'] = 'box'
            graph.append(graphviz_node)
            node_mapping[node] = graphviz_node
            if node.parent is not None:
                graphtools.GraphvizEdge().attach(
                    node_mapping[node.parent],
                    node_mapping[node],
                    )
        leaves = self._root_node.leaves
        offset = leaves[0].start_offset
        offset_subgraph = graphtools.GraphvizSubgraph(
            name='cluster_offsets',
            attributes={
                'style': 'rounded',
                },
            )
        graph.append(offset_subgraph)
        make_offset_node(offset, leaves[0])
        for one, two in sequencetools.Sequence(leaves).nwise():
            offset = one.stop_offset
            make_offset_node(offset, one, two)
        offset = leaves[-1].stop_offset
        make_offset_node(offset, leaves[-1], is_last=True)
        return graph

    def __hash__(self):
        r'''Hashes meter.
        '''
        return hash((type(self), self.rtm_format))

    def __iter__(self):
        r'''Iterates meter.

        ..  container:: example

            **Example 1.** Iterates ``5/4``:

            ::


                >>> meter = metertools.Meter((5, 4))
                >>> for x in meter:
                ...    x
                ...
                (NonreducedFraction(0, 4), NonreducedFraction(1, 4))
                (NonreducedFraction(1, 4), NonreducedFraction(2, 4))
                (NonreducedFraction(2, 4), NonreducedFraction(3, 4))
                (NonreducedFraction(0, 4), NonreducedFraction(3, 4))
                (NonreducedFraction(3, 4), NonreducedFraction(4, 4))
                (NonreducedFraction(4, 4), NonreducedFraction(5, 4))
                (NonreducedFraction(3, 4), NonreducedFraction(5, 4))
                (NonreducedFraction(0, 4), NonreducedFraction(5, 4))

        Yields pairs.
        '''
        def recurse(node):
            result = []
            for child in node:
                if isinstance(child, rhythmtreetools.RhythmTreeLeaf):
                    result.append(child)
                else:
                    result.extend(recurse(child))
            result.append(node)
            return result
        result = recurse(self.root_node)
        for x in result:
            start_offset = mathtools.NonreducedFraction(
                x.start_offset).with_denominator(self.denominator)
            stop_offset = mathtools.NonreducedFraction(
                x.stop_offset).with_denominator(self.denominator)
            yield start_offset, stop_offset

    def __str__(self):
        r'''Gets string representation of meter.

        ..  container:: example

            **Example 1.** Gets string representation of meters over ``8``:

            ::

                >>> for numerator in range(1, 9):
                ...     meter = metertools.Meter((numerator, 8))
                ...     print(str(meter))
                1/8
                2/8
                3/8
                4/8
                5/8
                6/8
                7/8
                8/8

        Returns string.
        '''
        return '{}/{}'.format(self.numerator, self.denominator)

    ### PRIVATE METHODS ###

    def _get_format_specification(self):
        return systemtools.FormatSpecification(
            client=self,
            repr_is_indented=False,
            storage_format_args_values=[self.rtm_format],
            storage_format_kwargs_names=[],
            )

    @staticmethod
    def _make_gridded_test_rhythm(grid_length, rhythm_number, denominator=16):
        r'''Make test rhythm number `rhythm_number` that fits `grid_length`.

        Returns selection of one or more possibly tied notes.

        ..  container:: example

            **Example 1.** The eight test rhythms that fit a length-``4``
            grid:

            ::

                >>> from abjad.tools.metertools import Meter
                >>> for rhythm_number in range(8):
                ...     notes = Meter._make_gridded_test_rhythm(
                ...         4, rhythm_number, denominator=4)
                ...     measure = Measure((4, 4), notes)
                ...     print('{}\t{}'.format(rhythm_number, str(measure)))
                ...
                0	Measure((4, 4), "c'1")
                1	Measure((4, 4), "c'2. c'4")
                2	Measure((4, 4), "c'2 c'4 c'4")
                3	Measure((4, 4), "c'2 c'2")
                4	Measure((4, 4), "c'4 c'4 c'2")
                5	Measure((4, 4), "c'4 c'4 c'4 c'4")
                6	Measure((4, 4), "c'4 c'2 c'4")
                7	Measure((4, 4), "c'4 c'2.")

        ..  container:: example

            **Example 2.** The sixteenth test rhythms for that a length-``5``
            grid:

            ::

                >>> for rhythm_number in range(16):
                ...     notes = Meter._make_gridded_test_rhythm(
                ...         5, rhythm_number, denominator=4)
                ...     measure = Measure((5, 4), notes)
                ...     print('{}\t{}'.format(rhythm_number, str(measure)))
                ...
                0	Measure((5, 4), "c'1 ~ c'4")
                1	Measure((5, 4), "c'1 c'4")
                2	Measure((5, 4), "c'2. c'4 c'4")
                3	Measure((5, 4), "c'2. c'2")
                4	Measure((5, 4), "c'2 c'4 c'2")
                5	Measure((5, 4), "c'2 c'4 c'4 c'4")
                6	Measure((5, 4), "c'2 c'2 c'4")
                7	Measure((5, 4), "c'2 c'2.")
                8	Measure((5, 4), "c'4 c'4 c'2.")
                9	Measure((5, 4), "c'4 c'4 c'2 c'4")
                10	Measure((5, 4), "c'4 c'4 c'4 c'4 c'4")
                11	Measure((5, 4), "c'4 c'4 c'4 c'2")
                12	Measure((5, 4), "c'4 c'2 c'2")
                13	Measure((5, 4), "c'4 c'2 c'4 c'4")
                14	Measure((5, 4), "c'4 c'2. c'4")
                15	Measure((5, 4), "c'4 c'1")

        Use for testing meter establishment.
        '''
        from abjad.tools import scoretools
        # check input
        assert mathtools.is_positive_integer(grid_length)
        assert isinstance(rhythm_number, int)
        assert mathtools.is_positive_integer_power_of_two(denominator)
        # find count of all rhythms that fit grid length
        rhythm_count = 2 ** (grid_length - 1)
        # read rhythm number cyclically to allow large and
        # negative rhythm numbers
        rhythm_number = rhythm_number % rhythm_count
        # find binary representation of rhythm
        binary_representation = mathtools.integer_to_binary_string(
            rhythm_number)
        binary_representation = binary_representation.zfill(grid_length)
        # partition binary representation of rhythm
        parts = sequencetools.Sequence(binary_representation).group_by()
        # find durations
        durations = [
            durationtools.Duration(len(part), denominator)
            for part in parts
            ]
        # make notes
        notes = scoretools.make_notes([0], durations)
        # return notes
        return notes

    ### PUBLIC METHODS ###

    @staticmethod
    def fit_meters_to_expr(
        argument,
        meters,
        denominator=32,
        discard_final_orphan_downbeat=True,
        maximum_run_length=None,
        starting_offset=None,
        ):
        r'''Finds the best-matching sequence of meters for the offsets
        contained in `argument`.

        ..  container:: example

            ::

                >>> meters = [(3, 4), (4, 4), (5, 4)]
                >>> meters = [metertools.Meter(_) for _ in meters]

        ..  container:: example

            **Example 1.** Matches a series of hypothetical ``4/4`` measures:

            ::

                >>> argument = [(0, 4), (4, 4), (8, 4), (12, 4), (16, 4)]
                >>> for x in metertools.Meter.fit_meters_to_expr(
                ...     argument, meters):
                ...     print(x.implied_time_signature)
                ...
                4/4
                4/4
                4/4
                4/4

        ..  container:: example

            **Example 2.** Matches a series of hypothetical ``5/4`` measures:

            ::

                >>> argument = [(0, 4), (3, 4), (5, 4), (10, 4), (15, 4), (20, 4)]
                >>> for x in metertools.Meter.fit_meters_to_expr(
                ...     argument, meters):
                ...     print(x.implied_time_signature)
                ...
                3/4
                4/4
                3/4
                5/4
                5/4

        Coerces offsets from `argument` via
        `MetricAccentKernel.count_offsets()`.

        Coerces Meters from `meters` via `MeterList`.

        Returns list.
        '''
        from abjad.tools import metertools
        session = metertools.MeterFittingSession(
            kernel_denominator=denominator,
            maximum_run_length=maximum_run_length,
            meters=meters,
            offset_counter=argument,
            )
        meters = session()
        return meters

    def generate_offset_kernel_to_denominator(
        self,
        denominator,
        normalize=True,
        ):
        r'''Generates a dictionary of all offsets in a meter up
        to `denominator`.

        Keys are the offsets and the values are the normalized weights of
        those offsets.

        ..  container:: example

            ::

                >>> meter = metertools.Meter((4, 4))
                >>> kernel = meter.generate_offset_kernel_to_denominator(8)
                >>> for offset, weight in sorted(kernel.kernel.items()):
                ...     print('{!s}\t{!s}'.format(offset, weight))
                ...
                0       3/16
                1/8     1/16
                1/4     1/8
                3/8     1/16
                1/2     1/8
                5/8     1/16
                3/4     1/8
                7/8     1/16
                1       3/16

        This is useful for testing how strongly a collection of offsets
        responds to a given meter.

        Returns dictionary.
        '''
        from abjad.tools import metertools
        assert mathtools.is_positive_integer_power_of_two(
            denominator // self.denominator)

        inventory = list(self.depthwise_offset_inventory)
        old_flag_count = durationtools.Duration(1, self.denominator).flag_count
        new_flag_count = durationtools.Duration(1, denominator).flag_count
        extra_depth = new_flag_count - old_flag_count
        for _ in range(extra_depth):
            old_offsets = inventory[-1]
            new_offsets = []
            for first, second in sequencetools.Sequence(old_offsets).nwise():
                new_offsets.append(first)
                new_offsets.append((first + second) / 2)
            new_offsets.append(old_offsets[-1])
            inventory.append(tuple(new_offsets))

        total = 0
        kernel = {}
        for offsets in inventory:
            for offset in offsets:
                if offset not in kernel:
                    kernel[offset] = 0
                kernel[offset] += 1
                total += 1

        if normalize:
            for offset, response in kernel.items():
                kernel[offset] = durationtools.Multiplier(response, total)

        return metertools.MetricAccentKernel(kernel)

    ### PUBLIC PROPERTIES ###

    @property
    def decrease_durations_monotonically(self):
        r'''Is true when meter divides large primes into collections of ``2``
        and ``3`` that decrease monotonically. Otherwise false.

        ..  container:: example

            **Example 1.** An asymmetric meter with beats arranged greatest to
            least:

            ::

                >>> meter = metertools.Meter(
                ...     (7, 4),
                ...     decrease_durations_monotonically=True,
                ...     )

            ::

                >>> meter.decrease_durations_monotonically
                True

            ::

                >>> print(meter.pretty_rtm_format)
                (7/4 (
                    (3/4 (
                        1/4
                        1/4
                        1/4))
                    (2/4 (
                        1/4
                        1/4))
                    (2/4 (
                        1/4
                        1/4))))

            This is default beahvior.

        ..  container:: example

            **Example 2.** The same asymmetric meter with unequal beats
            arranged least to greatest:

            ::

                >>> meter = metertools.Meter(
                ...     (7, 4),
                ...     decrease_durations_monotonically=False,
                ...     )

            ::

                >>> meter.decrease_durations_monotonically
                False

            ::

                >>> print(meter.pretty_rtm_format)
                (7/4 (
                    (2/4 (
                        1/4
                        1/4))
                    (2/4 (
                        1/4
                        1/4))
                    (3/4 (
                        1/4
                        1/4
                        1/4))))

        Returns true or false.
        '''
        return self._decrease_durations_monotonically

    @property
    def denominator(self):
        r'''Gets denominator of meter.

        ..  container:: example

            ::

                >>> meter = metertools.Meter((7, 4))
                >>> meter.denominator
                4

        Returns positive integer.
        '''
        return self._denominator

    @property
    def depthwise_offset_inventory(self):
        r'''Gets depthwise offset inventory of meter.

        ..  container:: example

            ::

                >>> for depth, offsets in enumerate(
                ...     meter.depthwise_offset_inventory):
                ...     print(depth, offsets)
                0 (Offset(0, 1), Offset(7, 4))
                1 (Offset(0, 1), Offset(3, 4), Offset(5, 4), Offset(7, 4))
                2 (Offset(0, 1), Offset(1, 4), Offset(1, 2), Offset(3, 4), Offset(1, 1), Offset(5, 4), Offset(3, 2), Offset(7, 4))

        Returns dictionary.
        '''
        inventory = []
        all_offsets = set()
        all_offsets.add(durationtools.Offset(self.numerator, self.denominator))
        for depth, nodes in sorted(self.root_node.depthwise_inventory.items()):
            for node in nodes:
                all_offsets.add(durationtools.Offset(node.start_offset))
            inventory.append(tuple(sorted(all_offsets)))
        return tuple(inventory)

    @property
    def duration(self):
        r'''Gets duration of meter.

        ..  container:: example

            ::

                >>> meter.duration
                Duration(7, 4)

        Returns duration.
        '''
        return durationtools.Duration(self.numerator, self.denominator)

    @property
    def implied_time_signature(self):
        r'''Gets implied time signature of meter.

        ..  container:: example

            ::

                >>> metertools.Meter((4, 4)).implied_time_signature
                TimeSignature((4, 4))

        Returns time signature.
        '''
        return indicatortools.TimeSignature(
            self.root_node.preprolated_duration)

    @property
    def is_compound(self):
        r'''Is true when meter is compound. Otherwise false.

        ..  container:: example

            **Example 1.** Compound meters written over ``4``:

            ::

                >>> for numerator in range(1, 13):
                ...     meter = metertools.Meter((numerator, 4))
                ...     string = True if meter.is_compound else ''
                ...     print(str(meter), string)
                ...
                1/4
                2/4
                3/4
                4/4
                5/4
                6/4     True
                7/4
                8/4
                9/4     True
                10/4
                11/4
                12/4    True

        ..  container:: example

            **Example 2.** Compound meters written over ``8``:

            ::

                >>> for numerator in range(1, 13):
                ...     meter = metertools.Meter((numerator, 8))
                ...     string = True if meter.is_compound else ''
                ...     print(str(meter), string)
                ...
                1/8
                2/8
                3/8
                4/8
                5/8
                6/8     True
                7/8
                8/8
                9/8     True
                10/8
                11/8
                12/8    True

        Compound meters defined equal to those meters with a numerator
        divisible by ``3`` (but not equal to ``3``).

        Returns true or false.
        '''
        if 3 in mathtools.divisors(self.numerator):
            if not self.numerator == 3:
                return True
        return False

    @property
    def is_simple(self):
        r'''Is true when meter is simple. Otherwise false.

        ..  container:: example

            **Example 1.** Simple meters written over ``4``:

            ::

                >>> for numerator in range(1, 13):
                ...     meter = metertools.Meter((numerator, 4))
                ...     string = True if meter.is_simple else ''
                ...     print(str(meter), string)
                ...
                1/4     True
                2/4     True
                3/4     True
                4/4     True
                5/4     True
                6/4
                7/4     True
                8/4     True
                9/4
                10/4    True
                11/4    True
                12/4

        ..  container:: example

            **Example 2.** Simple meters written over ``8``:

            ::

                >>> for numerator in range(1, 13):
                ...     meter = metertools.Meter((numerator, 8))
                ...     string = True if meter.is_simple else ''
                ...     print(str(meter), string)
                ...
                1/8     True
                2/8     True
                3/8     True
                4/8     True
                5/8     True
                6/8
                7/8     True
                8/8     True
                9/8
                10/8    True
                11/8    True
                12/8

        Simple meters defined equal to those meters with a numerator
        not divisible by ``3``.

        Meters with numerator equal to ``3`` are also defined as simple.

        Returns true or false.
        '''
        return not self.is_compound

    @property
    def numerator(self):
        r'''Gets numerator of meter.

        ..  container:: example

            ::

                >>> meter = metertools.Meter((7, 4))
                >>> meter.numerator
                7

        Returns positive integer.
        '''
        return self._numerator

    @property
    def pair(self):
        r'''Gets pair of numerator and denominator of meter.

        ..  container:: example

            ::

                >>> meter = metertools.Meter((6, 4))
                >>> meter.pair
                (6, 4)

        Returns pair.
        '''
        return (self.numerator, self.denominator)

    @property
    def preferred_boundary_depth(self):
        r'''Gets preferred boundary depth of meter.

        ..  container:: example

            **Example 1.** No preferred boundary depth:

            ::

                >>> metertools.Meter((6, 8)).preferred_boundary_depth is None
                True

        ..  container:: example

            **Example 2.** Customized preferred boundary depth:

            ::

                >>> meter = metertools.Meter(
                ...     (6, 8),
                ...     preferred_boundary_depth=1,
                ...     )
                >>> meter.preferred_boundary_depth
                1

        Used by ``mutate().rewrite_meter()``.

        Defaults to none.

        Set to integer or none.

        Returns integer or none.
        '''
        return self._preferred_boundary_depth

    @property
    def pretty_rtm_format(self):
        r'''Gets pretty RTM format of meter.

        ..  container:: example

            ::

                >>> meter = metertools.Meter((7, 4))
                >>> print(meter.pretty_rtm_format)
                (7/4 (
                    (3/4 (
                        1/4
                        1/4
                        1/4))
                    (2/4 (
                        1/4
                        1/4))
                    (2/4 (
                        1/4
                        1/4))))

        Returns string.
        '''
        return self.root_node.pretty_rtm_format

    @property
    def root_node(self):
        r'''Gets root node of meter.

        ..  container:: example

            ::

                >>> meter = metertools.Meter((7, 4))
                >>> print(format(meter.root_node))
                rhythmtreetools.RhythmTreeContainer(
                    children=(
                        rhythmtreetools.RhythmTreeContainer(
                            children=(
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=durationtools.Duration(1, 4),
                                    is_pitched=True,
                                    ),
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=durationtools.Duration(1, 4),
                                    is_pitched=True,
                                    ),
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=durationtools.Duration(1, 4),
                                    is_pitched=True,
                                    ),
                                ),
                            preprolated_duration=mathtools.NonreducedFraction(3, 4),
                            ),
                        rhythmtreetools.RhythmTreeContainer(
                            children=(
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=durationtools.Duration(1, 4),
                                    is_pitched=True,
                                    ),
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=durationtools.Duration(1, 4),
                                    is_pitched=True,
                                    ),
                                ),
                            preprolated_duration=mathtools.NonreducedFraction(2, 4),
                            ),
                        rhythmtreetools.RhythmTreeContainer(
                            children=(
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=durationtools.Duration(1, 4),
                                    is_pitched=True,
                                    ),
                                rhythmtreetools.RhythmTreeLeaf(
                                    preprolated_duration=durationtools.Duration(1, 4),
                                    is_pitched=True,
                                    ),
                                ),
                            preprolated_duration=mathtools.NonreducedFraction(2, 4),
                            ),
                        ),
                    preprolated_duration=mathtools.NonreducedFraction(7, 4),
                    )

        Returns rhythm tree node.
        '''
        return self._root_node

    @property
    def rtm_format(self):
        r'''Gets RTM format of meter.

        ..  container:: example

            ::

                >>> meter = metertools.Meter((7, 4))
                >>> meter.rtm_format
                '(7/4 ((3/4 (1/4 1/4 1/4)) (2/4 (1/4 1/4)) (2/4 (1/4 1/4))))'

        Returns string.
        '''
        return self._root_node.rtm_format



# -*- coding: utf-8 -*-
from abjad import *


def test_pitchtools_NumberedInterval_01():

    i = pitchtools.NumberedInterval(3)

    assert abs(i) == pitchtools.NumberedInterval(3)
    assert -i == pitchtools.NumberedInterval(-3)
    assert int(i) == 3
    assert float(i) == 3.0
